/// <reference path="../../typings/tsd.d.ts" />
define(["require", "exports", "common/messagePublisher", "common/appUrl", "common/oauthContext", "forge", "common/notifications/protractedCommandsDetector"], function (require, exports, messagePublisher, appUrl, oauthContext, forge, protractedCommandsDetector) {
    "use strict";
    /// Commands encapsulate a read or write operation to the database and support progress notifications and common AJAX related functionality.
    var commandBase = (function () {
        function commandBase() {
            this.oauthHandler = new oauthHandler(); //TODO: consider static handler?
        }
        commandBase.prototype.execute = function () {
            throw new Error("Execute must be overridden.");
        };
        commandBase.prototype.urlEncodeArgs = function (args) {
            return appUrl.urlEncodeArgs(args);
        };
        commandBase.prototype.getTimeToAlert = function (longWait) {
            return longWait ? 60000 : 9000;
        };
        commandBase.prototype.query = function (relativeUrl, args, resource, resultsSelector, options, timeToAlert) {
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            var ajax = this.ajax(relativeUrl, args, "GET", resource, options, timeToAlert);
            if (resultsSelector) {
                var task = $.Deferred();
                ajax.done(function (results, status, xhr) {
                    var transformedResults = resultsSelector(results);
                    task.resolve(transformedResults, status, xhr);
                });
                ajax.fail(function (request, status, error) {
                    task.reject(request, status, error);
                });
                return task;
            }
            else {
                return ajax;
            }
        };
        commandBase.prototype.head = function (relativeUrl, args, resource, resultsSelector) {
            var ajax = this.ajax(relativeUrl, args, "HEAD", resource);
            if (resultsSelector) {
                var task = $.Deferred();
                ajax.done(function (results, status, xhr) {
                    var allHeaders = xhr.getAllResponseHeaders();
                    if (allHeaders) {
                        var headersObject = {};
                        var headersArray = xhr.getAllResponseHeaders().trim().split(/\r?\n/);
                        for (var n = 0; n < headersArray.length; n++) {
                            var keyValue = headersArray[n].split(": ");
                            if (keyValue.length === 2) {
                                headersObject[keyValue[0]] = keyValue[1];
                            }
                        }
                        var transformedResults = resultsSelector(headersObject);
                        task.resolve(transformedResults);
                    }
                });
                ajax.fail(function (request, status, error) {
                    task.reject(request, status, error);
                });
                return task;
            }
            else {
                return ajax;
            }
        };
        commandBase.prototype.put = function (relativeUrl, args, resource, options, timeToAlert) {
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            return this.ajax(relativeUrl, args, "PUT", resource, options, timeToAlert);
        };
        commandBase.prototype.reset = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "RESET", resource, options);
        };
        commandBase.prototype.del = function (relativeUrl, args, resource, options, timeToAlert) {
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            return this.ajax(relativeUrl, args, "DELETE", resource, options, timeToAlert);
        };
        commandBase.prototype.post = function (relativeUrl, args, resource, options, timeToAlert) {
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            return this.ajax(relativeUrl, args, "POST", resource, options, timeToAlert);
        };
        commandBase.prototype.patch = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "PATCH", resource, options);
        };
        commandBase.prototype.evalJs = function (relativeUrl, args, resource, options) {
            return this.ajax(relativeUrl, args, "EVAL", resource, options);
        };
        commandBase.prototype.ajax = function (relativeUrl, args, method, resource, options, timeToAlert) {
            var _this = this;
            if (timeToAlert === void 0) { timeToAlert = 9000; }
            var originalArguments = arguments;
            var requestExecution = protractedCommandsDetector.instance.requestStarted(4000, timeToAlert);
            // ContentType:
            //
            // Can't use application/json in cross-domain requests, otherwise it 
            // issues OPTIONS preflight request first, which doesn't return proper 
            // headers(e.g.Etag header, etc.)
            // 
            // So, for GETs, we issue text/plain requests, which skip the OPTIONS
            // request and goes straight for the GET request.
            var contentType = method === "GET" ?
                "text/plain; charset=utf-8" :
                "application/json; charset=utf-8";
            var defaultOptions = {
                url: appUrl.forResourceQuery(resource) + relativeUrl,
                data: args,
                dataType: "json",
                contentType: contentType,
                type: method,
                headers: undefined,
                xhr: function () {
                    var xhr = new XMLHttpRequest();
                    xhr.upload.addEventListener("progress", function (evt) {
                        if (evt.lengthComputable) {
                            var percentComplete = (evt.loaded / evt.total) * 100;
                            if (percentComplete < 100) {
                                requestExecution.markProgress();
                            }
                            //TODO: use event
                            ko.postbox.publish("UploadProgress", percentComplete);
                        }
                    }, false);
                    return xhr;
                }
            };
            if (options) {
                for (var prop in options) {
                    defaultOptions[prop] = options[prop];
                }
            }
            if (oauthContext.apiKey()) {
                if (!defaultOptions.headers) {
                    var newHeaders = {};
                    defaultOptions.headers = newHeaders;
                }
                defaultOptions.headers["Has-Api-Key"] = "True";
            }
            if (oauthContext.authHeader()) {
                if (!defaultOptions.headers) {
                    var newHeaders = {};
                    defaultOptions.headers = newHeaders;
                }
                defaultOptions.headers["Authorization"] = oauthContext.authHeader();
            }
            var ajaxTask = $.Deferred();
            $.ajax(defaultOptions).always(function () {
                requestExecution.markCompleted();
            }).done(function (results, status, xhr) {
                ajaxTask.resolve(results, status, xhr);
            }).fail(function (request, status, error) {
                var dbBeingUpdated = request.getResponseHeader("Raven-Database-Load-In-Progress");
                if (dbBeingUpdated) {
                    ajaxTask.reject(request, status, error);
                }
                else if (request.status === 412 /* PreconditionFailed */ && oauthContext.apiKey()) {
                    _this.oauthHandler.handleOAuth(ajaxTask, request, function () { return _this.retryOriginalRequest(ajaxTask, originalArguments); });
                }
                else {
                    ajaxTask.reject(request, status, error);
                }
            });
            return ajaxTask.promise();
        };
        commandBase.prototype.retryOriginalRequest = function (task, orignalArguments) {
            this.ajax.apply(this, orignalArguments).done(function (results, status, xhr) {
                task.resolve(results, status, xhr);
            }).fail(function (request, status, error) {
                task.reject(request, status, error);
            });
        };
        commandBase.prototype.reportInfo = function (title, details) {
            messagePublisher.reportInfo(title, details);
        };
        commandBase.prototype.reportError = function (title, details, httpStatusText) {
            messagePublisher.reportError(title, details, httpStatusText);
        };
        commandBase.prototype.reportSuccess = function (title, details) {
            messagePublisher.reportSuccess(title, details);
        };
        commandBase.prototype.reportWarning = function (title, details, httpStatusText) {
            messagePublisher.reportWarning(title, details, httpStatusText);
        };
        return commandBase;
    }());
    // TODO: better place for this?
    commandBase.ravenClientVersion = '4.0.0.0';
    var oauthHandler = (function () {
        function oauthHandler() {
        }
        oauthHandler.prototype.handleOAuth = function (task, request, retry) {
            var _this = this;
            var oauthSource = request.getResponseHeader('OAuth-Source');
            // issue request to oauth source endpoint to get RSA exponent and modulus
            $.ajax({
                type: 'POST',
                url: oauthSource,
                headers: {
                    grant_type: 'client_credentials'
                }
            }).fail(function (request, status, error) {
                if (request.status !== 412 /* PreconditionFailed */) {
                    task.reject(request, status, error);
                }
                else {
                    var wwwAuth = request.getResponseHeader('WWW-Authenticate');
                    var tokens = wwwAuth.split(',');
                    var authRequest = {};
                    tokens.forEach(function (token) {
                        var eqPos = token.indexOf("=");
                        var kv = [token.substring(0, eqPos), token.substring(eqPos + 1)];
                        var m = kv[0].match(/[a-zA-Z]+$/g);
                        if (m) {
                            authRequest[m[0]] = kv[1];
                        }
                        else {
                            authRequest[kv[0]] = kv[1];
                        }
                    });
                    // form oauth request
                    var data = _this.objectToString({
                        exponent: authRequest.exponent,
                        modulus: authRequest.modulus,
                        data: _this.encryptAsymmetric(authRequest.exponent, authRequest.modulus, _this.objectToString({
                            "api key name": oauthContext.apiKeyName(),
                            "challenge": authRequest.challenge,
                            "response": _this.prepareResponse(authRequest.challenge)
                        }))
                    });
                    $.ajax({
                        type: 'POST',
                        url: oauthSource,
                        data: data,
                        headers: {
                            grant_type: 'client_credentials'
                        }
                    }).done(function (results) {
                        var resultsAsString = JSON.stringify(results, null, 0);
                        oauthContext.authHeader("Bearer " + resultsAsString.replace(/(\r\n|\n|\r)/gm, ""));
                        retry();
                    }).fail(function (request, status, error) {
                        task.reject(request, status, error);
                    });
                }
            });
        };
        oauthHandler.prototype.objectToString = function (input) {
            return $.map(input, function (value, key) { return key + "=" + value; }).join(',');
        };
        oauthHandler.prototype.prepareResponse = function (challenge) {
            var input = challenge + ";" + oauthContext.apiKeySecret();
            var md = forge.md.sha1.create();
            md.update(input);
            return forge.util.encode64(md.digest().getBytes());
        };
        oauthHandler.prototype.encryptAsymmetric = function (exponent, modulus, data) {
            var e = this.base64ToBigInt(exponent);
            var n = this.base64ToBigInt(modulus);
            var rsa = forge.pki.rsa;
            var publicKey = rsa.setPublicKey(n, e);
            var key = forge.random.getBytesSync(32);
            var iv = forge.random.getBytesSync(16);
            var keyAndIvEncrypted = publicKey.encrypt(key + iv, 'RSA-OAEP');
            var cipher = forge.cipher.createCipher('AES-CBC', key);
            cipher.start({ iv: iv });
            cipher.update(forge.util.createBuffer(data));
            cipher.finish();
            var encrypted = cipher.output;
            return forge.util.encode64(keyAndIvEncrypted + encrypted.data);
        };
        oauthHandler.prototype.base64ToBigInt = function (input) {
            input = forge.util.decode64(input);
            var hex = forge.util.bytesToHex(input);
            return new forge.jsbn.BigInteger(hex, 16);
        };
        return oauthHandler;
    }());
    return commandBase;
});

//# sourceMappingURL=commandBase.js.map
