/// <reference path="../../typings/tsd.d.ts" />
define(["require", "exports", "common/appUrl", "commands/auth/getSingleAuthTokenCommand", "common/messagePublisher"], function (require, exports, appUrl, getSingleAuthTokenCommand, messagePublisher) {
    "use strict";
    var abstractWebSocketClient = (function () {
        function abstractWebSocketClient(rs) {
            this.rs = rs;
            this.disposed = false;
            this.sentMessages = [];
            this.resourcePath = appUrl.forResourceQuery(this.rs);
            this.connectToWebSocketTask = $.Deferred();
            if ("WebSocket" in window) {
                this.connect(this.connectWebSocket);
            }
            else {
                //The browser doesn't support websocket
                //or we are in IE10 or IE11 and the server doesn't support WebSockets.
                //Anyway, at this point a warning message was already shown. 
                this.connectToWebSocketTask.reject();
            }
        }
        abstractWebSocketClient.prototype.connect = function (action, recoveringFromWebsocketFailure) {
            var _this = this;
            if (recoveringFromWebsocketFailure === void 0) { recoveringFromWebsocketFailure = false; }
            if (this.disposed) {
                if (!!this.connectToWebSocketTask)
                    this.connectToWebSocketTask.resolve();
                return;
            }
            if (!recoveringFromWebsocketFailure) {
                this.connectToWebSocketTask = $.Deferred();
            }
            new getSingleAuthTokenCommand(this.rs)
                .execute()
                .done(function (tokenObject) {
                action.call(_this, tokenObject);
            })
                .fail(function (e) {
                if (_this.disposed) {
                    _this.connectToWebSocketTask.reject();
                    return;
                }
                var error = !!e.responseJSON ? e.responseJSON.Error : e.responseText;
                if (e.status === 0) {
                    // Connection has closed so try to reconnect every 3 seconds.
                    setTimeout(function () { return _this.connect(action); }, 3 * 1000);
                }
                else if (e.status === 503 /* ServiceUnavailable */) {
                    // We're still loading the database, try to reconnect every 2 seconds.
                    setTimeout(function () { return _this.connect(action, true); }, 2 * 1000);
                }
                else if (e.status !== 403 /* Forbidden */) {
                    messagePublisher.reportError(error || "Failed to connect to changes", e.responseText, e.StatusText);
                    _this.connectToWebSocketTask.reject();
                }
            });
        };
        abstractWebSocketClient.prototype.onClose = function (e) {
            // empty
        };
        abstractWebSocketClient.prototype.connectWebSocket = function (token) {
            var _this = this;
            var connectionOpened = false;
            var wsProtocol = window.location.protocol === "https:" ? "wss://" : "ws://";
            var queryString = this.webSocketUrlFactory(token);
            var url = wsProtocol + window.location.host + this.resourcePath + queryString;
            this.webSocket = new WebSocket(url);
            this.webSocket.onmessage = function (e) { return _this.onMessage(e); };
            this.webSocket.onerror = function (e) {
                if (connectionOpened === false) {
                    _this.onError(e);
                }
            };
            this.webSocket.onclose = function (e) {
                _this.onClose(e);
                if (!e.wasClean) {
                    _this.onError(e);
                    // Connection has closed uncleanly, so try to reconnect.
                    _this.connect(_this.connectWebSocket);
                }
            };
            this.webSocket.onopen = function () {
                _this.onOpen();
                connectionOpened = true;
            };
        };
        abstractWebSocketClient.prototype.onOpen = function () {
            console.log("Connected to WebSocket changes API (" + this.connectionDescription + ")");
            this.reconnect();
        };
        abstractWebSocketClient.prototype.reconnect = function () {
            var _this = this;
            //TODO: don't send watch operations when server is restarted
            //send changes connection args after reconnecting
            this.sentMessages.forEach(function (args) { return _this.send(args.Command, args.Param, false); });
            if (abstractWebSocketClient.messageWasShownOnce) {
                messagePublisher.reportSuccess("Successfully reconnected to changes stream!");
                abstractWebSocketClient.messageWasShownOnce = false;
            }
        };
        abstractWebSocketClient.prototype.onError = function (e) {
            if (abstractWebSocketClient.messageWasShownOnce === false && !this.disposed) {
                messagePublisher.reportError("Changes stream was disconnected!", "Retrying connection shortly.");
                abstractWebSocketClient.messageWasShownOnce = true;
            }
        };
        abstractWebSocketClient.prototype.send = function (command, value, needToSaveSentMessages) {
            var _this = this;
            if (needToSaveSentMessages === void 0) { needToSaveSentMessages = true; }
            this.connectToWebSocketTask.done(function () {
                var args = {
                    Command: command
                };
                if (value !== undefined) {
                    args.Param = value;
                }
                var payload = JSON.stringify(args, null, 2);
                if (!_this.closingOrClosed() || !_this.isUnwatchCommand(command)) {
                    _this.webSocket.send(payload);
                }
                _this.saveSentMessages(needToSaveSentMessages, command, args);
            });
        };
        abstractWebSocketClient.prototype.closingOrClosed = function () {
            var state = this.webSocket.readyState;
            return WebSocket.CLOSED === state || WebSocket.CLOSING === state;
        };
        abstractWebSocketClient.prototype.isUnwatchCommand = function (command) {
            return command.slice(0, 2) === "un";
        };
        abstractWebSocketClient.prototype.saveSentMessages = function (needToSaveSentMessages, command, args) {
            if (needToSaveSentMessages) {
                if (this.isUnwatchCommand(command)) {
                    var commandName_1 = command.slice(2, command.length);
                    this.sentMessages = this.sentMessages.filter(function (msg) { return msg.Command !== commandName_1; });
                }
                else {
                    this.sentMessages.push(args);
                }
            }
        };
        abstractWebSocketClient.prototype.fireEvents = function (events, param, filter) {
            for (var i = 0; i < events.length; i++) {
                if (filter(param)) {
                    events[i].fire(param);
                }
            }
        };
        abstractWebSocketClient.prototype.dispose = function () {
            var _this = this;
            this.disposed = true;
            this.connectToWebSocketTask.done(function () {
                if (_this.webSocket && _this.webSocket.readyState === abstractWebSocketClient.readyStateOpen) {
                    console.log("Disconnecting from WebSocket (" + _this.connectionDescription + ")");
                    _this.webSocket.close();
                }
            });
        };
        abstractWebSocketClient.prototype.getResource = function () {
            return this.rs;
        };
        return abstractWebSocketClient;
    }());
    abstractWebSocketClient.readyStateOpen = 1;
    abstractWebSocketClient.messageWasShownOnce = false;
    return abstractWebSocketClient;
});

//# sourceMappingURL=abstractWebSocketClient.js.map
