/// <reference path="../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var extensions = (function () {
        function extensions() {
        }
        extensions.install = function () {
            extensions.installObservableExtensions();
            extensions.installStorageExtension();
            extensions.installBindingHandlers();
            extensions.installJqueryExtensions();
            extensions.configureValidation();
            // Want Intellisense for your extensions?
            // Go to extensionInterfaces.ts and add the function signature there.
        };
        extensions.configureValidation = function () {
            ko.validation.rules['base64'] = {
                validator: function (val) {
                    var base64regex = /^([0-9a-zA-Z+/]{4})*(([0-9a-zA-Z+/]{2}==)|([0-9a-zA-Z+/]{3}=))?$/;
                    return !val || base64regex.test(val);
                },
                message: 'Invaild base64 string.'
            };
            ko.validation.init({
                errorElementClass: 'has-error',
                errorMessageClass: 'help-block',
                decorateInputElement: true
            });
        };
        extensions.installObservableExtensions = function () {
            var subscribableFn = ko.subscribable.fn;
            subscribableFn.distinctUntilChanged = function () {
                var observable = this;
                var matches = ko.observable();
                var lastMatch = observable();
                observable.subscribe(function (val) {
                    if (val !== lastMatch) {
                        lastMatch = val;
                        matches(val);
                    }
                });
                return matches;
            };
            subscribableFn.throttle = function (throttleTimeMs) {
                var observable = this;
                return ko.pureComputed(function () { return observable(); }).extend({ throttle: throttleTimeMs });
            };
            subscribableFn.toggle = function () {
                var observable = this;
                observable(!observable());
                return observable;
            };
        };
        extensions.installStorageExtension = function () {
            Storage.prototype.getObject = function (key) {
                var value = this.getItem(key);
                return value && JSON.parse(value);
            };
            Storage.prototype.setObject = function (key, value) {
                this.setItem(key, ko.toJSON(value));
            };
        };
        extensions.installBindingHandlers = function () {
            ko.bindingHandlers["scrollTo"] = {
                update: function (element, valueAccessor) {
                    if (valueAccessor()) {
                        var $container = $("#page-host-root");
                        var scrollTop = $container.scrollTop();
                        var scrollBottom = scrollTop + $container.height();
                        var $element = $(element);
                        var elementTop = $element.position().top;
                        var elementBottom = elementTop + $element.height();
                        // Scroll vertically only if element is outside of viewport 
                        if ((elementTop < scrollTop) || (elementBottom > scrollBottom)) {
                            $container.scrollTop(elementTop);
                        }
                    }
                }
            };
            ko.bindingHandlers["collapse"] = {
                init: function (element, valueAccessor) {
                    var value = valueAccessor();
                    var valueUnwrapped = ko.unwrap(value);
                    var $element = $(element);
                    $element
                        .addClass('collapse')
                        .collapse({
                        toggle: valueUnwrapped
                    });
                },
                update: function (element, valueAccessor) {
                    var value = valueAccessor();
                    var valueUnwrapped = ko.unwrap(value);
                    $(element).collapse(valueUnwrapped ? "show" : "hide");
                }
            };
            ko.bindingHandlers["numericValue"] = {
                init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    var underlyingObservable = valueAccessor();
                    var interceptor = ko.computed({
                        read: underlyingObservable,
                        write: function (value) {
                            if (!isNaN(value)) {
                                underlyingObservable(parseFloat(value));
                            }
                        },
                        disposeWhenNodeIsRemoved: element
                    });
                    ko.bindingHandlers.value.init(element, function () { return interceptor; }, allBindingsAccessor, viewModel, bindingContext);
                },
                update: ko.bindingHandlers.value.update
            };
            ko.bindingHandlers["customValidity"] = {
                update: function (element, valueAccessor) {
                    var errorMessage = ko.unwrap(valueAccessor()); //unwrap to get subscription
                    element.setCustomValidity(errorMessage);
                }
            };
            ko.bindingHandlers["bsChecked"] = {
                init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                    var value = valueAccessor();
                    var newValueAccessor = function () {
                        return {
                            change: function () {
                                value(element.value);
                            }
                        };
                    };
                    if ($(element).val() === ko.unwrap(valueAccessor())) {
                        $(element).closest(".btn").button("toggle");
                    }
                    ko.bindingHandlers.event.init(element, newValueAccessor, allBindingsAccessor, viewModel, bindingContext);
                }
            };
            var key = "_my_init_key_";
            ko.bindingHandlers["updateHighlighting"] = {
                init: function (element) {
                    ko.utils.domData.set(element, key, true);
                },
                update: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
                    var value = valueAccessor();
                    var isInit = ko.utils.domData.get(element, key);
                    var data = ko.dataFor(element);
                    var skip = !!data.isAllDocuments || !!data.isSystemDocuments || !!data.isAllGroupsGroup;
                    if (skip === false && isInit === false) {
                        $($(element).parents("li")).highlight();
                    }
                    else {
                        ko.utils.domData.set(element, key, false);
                    }
                }
            };
            ko.bindingHandlers["checkboxTriple"] = {
                update: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
                    var checkboxValue = ko.unwrap(valueAccessor());
                    switch (checkboxValue) {
                        case 2 /* Checked */:
                            element.checked = true;
                            element.readOnly = false;
                            element.indeterminate = false;
                            break;
                        case 1 /* SomeChecked */:
                            element.readOnly = true;
                            element.indeterminate = true;
                            element.checked = false;
                            break;
                        case 0 /* UnChecked */:
                            element.checked = false;
                            element.readOnly = false;
                            element.indeterminate = false;
                            break;
                    }
                }
            };
        };
        extensions.installJqueryExtensions = function () {
            //TODO do we need it?
            jQuery.fn.highlight = function () {
                $(this).each(function () {
                    var el = $(this);
                    el.before("<div/>");
                    el.prev()
                        .width(el.width())
                        .height(el.height())
                        .css({
                        "position": "absolute",
                        "background-color": "#ffff99",
                        "opacity": ".9"
                    })
                        .fadeOut(1500);
                });
            };
        };
        return extensions;
    }());
    return extensions;
});

//# sourceMappingURL=extensions.js.map
