/// <reference path="../../../../typings/tsd.d.ts" />
define(["require", "exports"], function (require, exports) {
    "use strict";
    var gapFinder = (function () {
        function gapFinder(dateRanges, minGapTime) {
            this.minGapTime = minGapTime;
            this.domain = [];
            this.gapsPositions = [];
            this.sortDateRanges(dateRanges);
            this.computeGaps(dateRanges, minGapTime);
        }
        gapFinder.prototype.createScale = function (totalWidth, paddingBetweenGaps) {
            return gapFinder.createScaleInternal(totalWidth, paddingBetweenGaps, this.domain);
        };
        gapFinder.prototype.trimmedScale = function (timeRange, totalWidth, paddingBetweenGaps) {
            var trimmedDomain = gapFinder.trimDomain(this.domain, timeRange[0], timeRange[1]);
            return gapFinder.createScaleInternal(totalWidth, paddingBetweenGaps, trimmedDomain);
        };
        /**
        *  Returns helper function which translates milliseconds to pixels in scale with gaps
        */
        gapFinder.extentGeneratorForScaleWithGaps = function (scale) {
            var totalTime = gapFinder.calculateTotalTime(scale.domain());
            var totalRangeWidth = gapFinder.calculateRangeWidthExceptGaps(scale.range());
            return function (millis) { return millis * totalRangeWidth / totalTime; };
        };
        gapFinder.prototype.sortDateRanges = function (dateRanges) {
            dateRanges.sort(function (a, b) { return d3.ascending(a[0].getTime(), b[0].getTime()); });
        };
        gapFinder.prototype.computeGaps = function (dateRanges, minGapTime) {
            if (dateRanges.length > 0) {
                var s = dateRanges[0][0];
                var e = dateRanges[0][1];
                this.minTime = s;
                this.maxTime = e;
                for (var i = 1; i < dateRanges.length; i++) {
                    var newRange = dateRanges[i];
                    if (this.minTime > newRange[0]) {
                        this.minTime = newRange[0];
                    }
                    if (this.maxTime < newRange[1]) {
                        this.maxTime = newRange[1];
                    }
                    if (newRange[0].getTime() > e.getTime() + minGapTime) {
                        this.pushRegion([s, e]);
                        s = newRange[0];
                    }
                    if (newRange[1].getTime() > e.getTime()) {
                        e = newRange[1];
                    }
                }
                this.pushRegion([s, e]);
            }
        };
        gapFinder.prototype.pushRegion = function (region) {
            if (this.domain.length > 0) {
                // since domain is not empty push gap
                var lastEnd = _.last(this.domain);
                var gapStart = region[0];
                this.gapsPositions.push({
                    start: lastEnd,
                    durationInMillis: gapStart.getTime() - lastEnd.getTime()
                });
            }
            this.domain.push(region[0]);
            this.domain.push(new Date(region[1].getTime() + 1)); // add 1 extra millisecond to avoid issues with rounding time
        };
        gapFinder.calculateTotalTime = function (domain) {
            var totalTime = 0;
            for (var i = 0; i < domain.length / 2; i++) {
                var s = domain[2 * i];
                var e = domain[2 * i + 1];
                var periodDuration = e.getTime() - s.getTime();
                totalTime += periodDuration;
            }
            return totalTime;
        };
        gapFinder.calculateRangeWidthExceptGaps = function (input) {
            var width = 0;
            for (var i = 0; i < input.length / 2; i++) {
                var s = input[2 * i];
                var e = input[2 * i + 1];
                var periodDuration = e - s;
                width += periodDuration;
            }
            return width;
        };
        gapFinder.createScaleInternal = function (totalWidth, paddingBetweenGaps, domain) {
            var totalTime = gapFinder.calculateTotalTime(domain);
            var gapsCount = domain.length / 2 - 1;
            var widthExceptGrapsPadding = totalWidth - paddingBetweenGaps * gapsCount;
            var extentFunc = function (period) { return period * widthExceptGrapsPadding / totalTime; };
            var currentX = 0;
            var range = [];
            for (var i = 0; i < domain.length / 2; i++) {
                var s = domain[2 * i];
                var e = domain[2 * i + 1];
                var periodDuration = e.getTime() - s.getTime();
                var scaleExtent = extentFunc(periodDuration);
                range.push(currentX);
                currentX += scaleExtent;
                range.push(currentX);
                currentX += paddingBetweenGaps;
            }
            return d3.time.scale()
                .range(range)
                .domain(domain);
        };
        gapFinder.trimDomain = function (domain, start, end) {
            var result = [];
            // requested: |------|
            // items:              |----|  |-----|
            // result:    |------|
            if (end.getTime() < domain[0].getTime()) {
                result.push(start);
                result.push(end);
                return result;
            }
            // requested:                |------|
            // items:    |----|  |-----|
            // result:                   |------|
            if (start.getTime() > _.last(domain).getTime()) {
                result.push(start);
                result.push(end);
                return result;
            }
            var canSnapToLeft = true;
            for (var i = 0; i < domain.length / 2; i++) {
                var s = domain[2 * i];
                var e = domain[2 * i + 1];
                // item ends before requested start 
                // skip it
                if (e.getTime() < start.getTime()) {
                    continue;
                }
                // item starts after requested end
                // skip it
                if (s.getTime() > end.getTime()) {
                    continue;
                }
                // requested range falls into item range
                // show only requested range
                // requested:      |----|
                // item:       |------------|
                // result:         |----|
                if (s.getTime() < start.getTime() && e.getTime() > end.getTime()) {
                    result.push(start);
                    result.push(end);
                    continue;
                }
                var effectiveStart = s;
                var effectiveEnd = e;
                if (canSnapToLeft && start.getTime() < s.getTime()) {
                    effectiveStart = start;
                }
                else if (start.getTime() > s.getTime()) {
                    effectiveStart = start;
                }
                // if last item or next item starts after requested end
                var isLastMatchingItem = e.getTime() === _.last(domain).getTime() || end.getTime() < domain[2 * i + 2].getTime();
                if (end.getTime() < e.getTime()) {
                    effectiveEnd = end;
                }
                else if (end.getTime() > e.getTime() && isLastMatchingItem) {
                    effectiveEnd = end;
                }
                result.push(effectiveStart);
                result.push(effectiveEnd);
                canSnapToLeft = false;
            }
            return result;
        };
        return gapFinder;
    }());
    return gapFinder;
});

//# sourceMappingURL=gapFinder.js.map
