define(["require", "exports", "common/constants/events", "common/notifications/watchedOperation", "common/notifications/notificationCenterPersistanceStorage", "common/messagePublisher", "common/changesContext", "commands/operations/getOperationCommand", "commands/operations/dismissOperationCommand", "commands/operations/getOperationsCommand", "commands/operations/killOperationCommand"], function (require, exports, EVENTS, watchedOperation, notificationCenterPersistanceStorage, messagePublisher, changesContext, getOperationCommand, dismissOperationCommand, getOperationsCommand, killOperationCommand) {
    "use strict";
    var notificationCenterOperations = (function () {
        function notificationCenterOperations() {
            var _this = this;
            this.watchedOperations = ko.observableArray([]);
            this.serverTime = ko.observable();
            this.activeChangesApi = ko.observable();
            this.storage = new notificationCenterPersistanceStorage(this.serverTime);
            this.activeChangesApi = changesContext.default.currentResourceChangesApi;
            ko.postbox.subscribe(EVENTS.ChangesApi.Reconnected, function () { return _this.onReconnect(); });
            $(window).bind("storage", function (event) { return _this.onStorageEvent(event); });
        }
        notificationCenterOperations.prototype.onStorageEvent = function (event) {
            var _this = this;
            var storageEvent = event.originalEvent;
            var rs = this.activeChangesApi().getResource();
            if (!this.storage.isStorageKeyMatch(rs, storageEvent)) {
                return;
            }
            var oldValue = JSON.parse(storageEvent.oldValue);
            var oldOperations = oldValue ? oldValue.Operations : [];
            var newValue = JSON.parse(storageEvent.newValue);
            var newOperations = newValue ? newValue.Operations : [];
            var newOperationIds = newOperations.filter(function (x) { return oldOperations.indexOf(x) < 0; });
            var dismissedOperationIds = oldOperations.filter(function (x) { return newOperations.indexOf(x) < 0; });
            newOperationIds.forEach(function (id) {
                _this.monitorOperation(rs, id, null);
            });
            dismissedOperationIds.forEach(function (id) {
                _this.dismissOperation(id, false);
            });
        };
        notificationCenterOperations.prototype.onReconnect = function () {
            var _this = this;
            var changes = this.activeChangesApi();
            var rs = changes.getResource();
            this.serverTime(changes.serverStartTime());
            this.watchedOperations.removeAll();
            var operationIds = this.storage.loadOperations(changes.getResource());
            new getOperationsCommand(rs)
                .execute()
                .done(function (operations) {
                operationIds.forEach(function (operationId) {
                    var matchedOp = operations.find(function (x) { return x.Id === operationId; });
                    if (matchedOp) {
                        _this.subscribeForOperation(operationId, function (n) { return _this.onStatus(rs, n.OperationId, n.State); }); //TODO: shold we pass task and onProgres here?
                        _this.onOperationInfo(rs, matchedOp, operationId, null, null); //TODO: pass task, and on progress here
                    }
                });
            })
                .fail(function (response) { return messagePublisher.reportError("Failed to get running tasks", response.responseText, response.statusText); });
        };
        notificationCenterOperations.prototype.monitorOperation = function (rs, operationId, onProgress) {
            var _this = this;
            if (onProgress === void 0) { onProgress = null; }
            var task = $.Deferred();
            this.subscribeForOperation(operationId, function (notification) { return _this.onStatus(rs, notification.OperationId, notification.State, onProgress, task); });
            this.storage.saveOperations(rs, this.getOperationIds());
            this.fetchOperationWithRetries(rs, operationId, 5)
                .done(function (operation) { return _this.onOperationInfo(rs, operation, operationId, onProgress, task); })
                .fail(function (response) { return messagePublisher.reportError("Failed to get operation", response.responseText, response.statusText); });
            return task.promise();
        };
        notificationCenterOperations.prototype.subscribeForOperation = function (operationId, onChange) {
            var disposeHandle = this.activeChangesApi().watchOperation(operationId, onChange);
            var watchedOp = new watchedOperation(operationId, disposeHandle);
            this.watchedOperations.unshift(watchedOp);
        };
        notificationCenterOperations.prototype.killOperation = function (operationId) {
            var rs = this.activeChangesApi().getResource();
            new killOperationCommand(rs, operationId).execute()
                .done(); //TODO mark as kill sent 
        };
        notificationCenterOperations.prototype.dismissOperation = function (operationId, saveOperations) {
            if (saveOperations === void 0) { saveOperations = true; }
            var watchedOperation = this.getWatchedOperationById(operationId);
            var rs = this.activeChangesApi().getResource();
            if (watchedOperation) {
                new dismissOperationCommand(rs, operationId)
                    .execute(); // we don't care about successful completion here
                this.watchedOperations.remove(watchedOperation);
                if (saveOperations) {
                    this.storage.saveOperations(rs, this.getOperationIds());
                }
            }
        };
        notificationCenterOperations.prototype.onOperationInfo = function (rs, operation, operationId, onProgress, task) {
            if (onProgress === void 0) { onProgress = null; }
            var watchedOperation = this.getWatchedOperationById(operationId);
            if (watchedOperation) {
                watchedOperation.onInfoFetched(operation);
            }
            this.onStatus(rs, operationId, operation.State, onProgress, task);
        };
        notificationCenterOperations.prototype.getWatchedOperationById = function (operationId) {
            return this.watchedOperations().find(function (x) { return x.operationId === operationId; });
        };
        notificationCenterOperations.prototype.onStatus = function (rs, operationId, state, onProgress, task) {
            if (onProgress === void 0) { onProgress = null; }
            if (task === void 0) { task = null; }
            var watchedOperation = this.getWatchedOperationById(operationId);
            if (watchedOperation) {
                watchedOperation.onStatus(state);
            }
            // now handle optional parameters
            if (state.Status === "InProgress") {
                if (onProgress) {
                    onProgress(state.Progress);
                }
            }
            else {
                if (task) {
                    if (state.Status === "Completed") {
                        task.resolve(state.Result);
                    }
                    else {
                        task.reject(state.Result);
                    }
                }
            }
        };
        notificationCenterOperations.prototype.fetchOperationWithRetries = function (rs, operationId, retries) {
            if (retries === void 0) { retries = 3; }
            var task = $.Deferred();
            this.fetchOperation(rs, operationId, retries, task);
            return task;
        };
        notificationCenterOperations.prototype.fetchOperation = function (rs, operationId, retries, task) {
            var _this = this;
            new getOperationCommand(rs, operationId)
                .execute()
                .done(function (op) { return task.resolve(op); })
                .fail(function (response) {
                if (retries === 0) {
                    task.reject(response);
                    return;
                }
                setTimeout(function () { return _this.fetchOperation(rs, operationId, retries - 1, task); }, 500);
            });
        };
        notificationCenterOperations.prototype.getOperationIds = function () {
            return this.watchedOperations().map(function (x) { return x.operationId; });
        };
        return notificationCenterOperations;
    }());
    return notificationCenterOperations;
});

//# sourceMappingURL=notificationCenterOperations.js.map
