/// <reference path="../../../typings/tsd.d.ts"/>
define(["require", "exports", "durandal/app", "common/constants/events", "plugins/router", "common/shell/menu/leafMenuItem"], function (require, exports, app, EVENTS, router, leafMenuItem) {
    "use strict";
    var menu = (function () {
        function menu(items) {
            var _this = this;
            this.activeItem = ko.observable(null);
            this.deepestOpenItem = ko.observable(null);
            this.level = ko.computed(function () {
                var item = _this.deepestOpenItem();
                return item ? item.depth() + 1 : 0;
            });
            this.type = 'menu';
            this.items = ko.observable(null);
            this.itemsFlattened = ko.pureComputed(function () {
                return _this.flattenItems(_this.items() || []);
            });
            this.routeToItemCache = ko.pureComputed(function () {
                return _this.itemsFlattened()
                    .reduce(function (result, next) {
                    if (next.type !== 'leaf') {
                        return result;
                    }
                    var route = next.route;
                    if (typeof (route) === 'string') {
                        cacheItem(route, next);
                    }
                    else if (Array.isArray(route)) {
                        route.filter(function (x) { return !!x; })
                            .forEach(function (r) { return cacheItem(r, next); });
                    }
                    else {
                        throw new Error("Unknown route type: " + route + " " + typeof (route));
                    }
                    return result;
                    function cacheItem(route, item) {
                        var regex = routeStringToRegExp(route);
                        if (result.has(regex)) {
                            throw new Error("Duplicate menu item '" + item.title + "' for route: " + route + ".");
                        }
                        result.set(regex, item);
                    }
                }, new Map());
            });
            this.registeredRoutes = ko.computed(function () {
                return Array.from(_this.routeToItemCache().keys());
            });
            this.items(items);
        }
        menu.prototype.handleIntermediateItemClick = function ($data, $event) {
            var item = $data.item;
            if (item.isOpen()) {
                item.close();
                return;
            }
            var currentItem = this.deepestOpenItem();
            if (currentItem && currentItem !== item) {
                currentItem.close();
            }
            this.open(item, $event);
        };
        menu.prototype.open = function (item, $event) {
            $event.stopPropagation();
            item.open();
            this.deepestOpenItem(item);
            if (item.path()) {
                if ($event.ctrlKey) {
                    window.open(item.path());
                }
                else {
                    router.navigate(item.path());
                }
            }
        };
        menu.prototype.navigate = function ($data, $event) {
            if (this.shouldOpenAsDialog($data)) {
                var leafItem = $data;
                require([leafItem.moduleId], function (viewModel) {
                    app.showBootstrapDialog(new viewModel);
                });
            }
            else {
                var a = $event.currentTarget;
                if ($event.ctrlKey) {
                    window.open(a.href);
                }
                else {
                    router.navigate(a.href);
                }
            }
        };
        menu.prototype.shouldOpenAsDialog = function ($data) {
            return $data instanceof leafMenuItem && $data.openAsDialog;
        };
        menu.prototype.back = function ($data, $event) {
            var item = $data.item;
            $event.stopPropagation();
            item.isOpen(false);
            this.deepestOpenItem(item.parent());
            $($event.target)
                .closest('.level')
                .removeClass('level-show');
        };
        menu.prototype.update = function (items) {
            this.items(items);
            this.setActiveMenuItem();
        };
        menu.prototype.handleLevelClick = function ($data, $event) {
            $event.stopPropagation();
            var $targetLevel = $($event.currentTarget);
            var targetLevelValue = parseInt($targetLevel.attr('data-level'));
            if (this.level() === targetLevelValue) {
                return;
            }
            var itemAtCurrentLevel = this.deepestOpenItem();
            while (this.level() > targetLevelValue) {
                itemAtCurrentLevel.isOpen(false);
                itemAtCurrentLevel = itemAtCurrentLevel.parent();
                this.deepestOpenItem(itemAtCurrentLevel);
            }
        };
        menu.prototype.initialize = function () {
            var _this = this;
            this.$mainMenuLevels = $('#main-menu [data-level]');
            router.on('router:navigation:complete', function () {
                _this.setActiveMenuItem();
            });
            this.setActiveMenuItem();
        };
        menu.prototype.setActiveMenuItem = function () {
            for (var _i = 0, _a = this.itemsFlattened(); _i < _a.length; _i++) {
                var item_1 = _a[_i];
                if (item_1.type === 'intermediate') {
                    item_1.isOpen(false);
                }
            }
            var fragment = router.activeInstruction().fragment;
            var matchingRoute = this.registeredRoutes()
                .find(function (routeRegex) { return routeRegex.test(fragment); });
            if (!matchingRoute) {
                return;
            }
            var item = this.routeToItemCache().get(matchingRoute);
            if (item.nav) {
                // Highlight/Activate current item
                this.activeItem(item);
            }
            else if (item.itemRouteToHighlight) {
                // Highlight/Activate a different menu item
                var matchingRoute_1 = this.registeredRoutes()
                    .find(function (routeRegex) { return routeRegex.test(item.itemRouteToHighlight); });
                var itemToActivate = this.routeToItemCache().get(matchingRoute_1);
                if (itemToActivate.nav) {
                    this.activeItem(itemToActivate);
                }
            }
            this.setLevelToActiveItem();
        };
        menu.prototype.setLevelToActiveItem = function () {
            var active = this.activeItem();
            if (!active) {
                return;
            }
            var current = active.parent();
            this.deepestOpenItem(current);
            while (current) {
                current.isOpen(true);
                current = current.parent();
            }
        };
        menu.prototype.getDeepestOpenLevelElement = function () {
            var _this = this;
            return this.$mainMenuLevels.find('.level-show')
                .toArray()
                .reduce(function (result, nextEl) {
                if (!result) {
                    return nextEl;
                }
                var resultLevel = _this.parseLevel(result);
                var curLevel = _this.parseLevel(nextEl);
                if (resultLevel > curLevel) {
                    return result;
                }
                return nextEl;
            }, null);
        };
        menu.prototype.parseLevel = function (el) {
            return parseInt(el.dataset['level']);
        };
        menu.prototype.emitLevelChanged = function () {
            ko.postbox.publish(EVENTS.Menu.LevelChanged, this.level);
        };
        menu.prototype.getItemsFlattened = function () {
            return this.flattenItems(this.items());
        };
        menu.prototype.flattenItems = function (items) {
            return items.reduce(function (result, next) {
                addToResult(result, next);
                return result;
            }, []);
            function addToResult(result, item) {
                if (item.type === 'intermediate') {
                    item.children
                        .forEach(function (x) { return addToResult(result, x); });
                }
                result.push(item);
            }
        };
        return menu;
    }());
    /*
        The following code has been extracted from durandal router plugin since it was private - I needed the logic transforming the route into a regular expression. Here's the original location: https://github.com/BlueSpire/Durandal/blob/master/src/plugins/js/router.js#L23
    */
    var optionalParam = /\((.*?)\)/g;
    var namedParam = /(\(\?)?:\w+/g;
    var splatParam = /\*\w+/g;
    var escapeRegExp = /[\-{}\[\]+?.,\\\^$|#\s]/g;
    var routesAreCaseSensitive = false;
    function routeStringToRegExp(routeString) {
        routeString = routeString.replace(escapeRegExp, '\\$&')
            .replace(optionalParam, '(?:$1)?')
            .replace(namedParam, function (match, optional) {
            return optional ? match : '([^\/]+)';
        })
            .replace(splatParam, '(.*?)');
        return new RegExp('^' + routeString + '$', routesAreCaseSensitive ? undefined : 'i');
    }
    return menu;
});

//# sourceMappingURL=menu.js.map
