define(["require", "exports", "plugins/router", "common/constants/events", "models/resources/database", "models/filesystem/filesystem", "models/counter/counterStorage", "models/timeSeries/timeSeries", "common/changesContext", "commands/resources/getResourcesCommand", "commands/resources/getResourceCommand", "common/appUrl", "common/messagePublisher", "common/shell/activeResourceTracker"], function (require, exports, router, EVENTS, database, filesystem, counterStorage, timeSeries, changesContext, getResourcesCommand, getResourceCommand, appUrl, messagePublisher, activeResourceTracker) {
    "use strict";
    var resourcesManager = (function () {
        function resourcesManager() {
            var _this = this;
            this.initialized = $.Deferred();
            this.activeResourceTracker = activeResourceTracker.default;
            this.changesContext = changesContext.default;
            this.resourceToActivate = ko.observable();
            this.resources = ko.observableArray([]);
            this.databases = ko.computed(function () { return _this.resources().filter(function (x) { return x instanceof database; }); });
            this.fileSystems = ko.computed(function () { return _this.resources().filter(function (x) { return x instanceof filesystem; }); });
            this.counterStorages = ko.computed(function () { return _this.resources().filter(function (x) { return x instanceof counterStorage; }); });
            this.timeSeries = ko.computed(function () { return _this.resources().filter(function (x) { return x instanceof timeSeries; }); });
            this.changesContext.connectGlobalChangesApi()
                .done(function () {
                _this.changesContext.globalChangesApi().watchReconnect(function () { return _this.refreshResources(); });
            });
            ko.postbox.subscribe(EVENTS.ChangesApi.Reconnected, function (rs) { return _this.reloadDataAfterReconnection(rs); });
            ko.postbox.subscribe(EVENTS.Resource.Activate, function (_a) {
                var resource = _a.resource;
                if (resource instanceof database) {
                    return _this.activateDatabase(resource);
                }
                else if (resource instanceof filesystem) {
                    return _this.activateFileSystem(resource);
                }
                else if (resource instanceof counterStorage) {
                }
                else if (resource instanceof timeSeries) {
                }
                throw new Error("Invalid resource type " + resource.type);
            });
        }
        resourcesManager.prototype.getDatabaseByName = function (name) {
            if (!name) {
                return null;
            }
            return this
                .databases()
                .find(function (x) { return name.toLowerCase() === x.name.toLowerCase(); });
        };
        resourcesManager.prototype.getFileSystemByName = function (name) {
            if (!name) {
                return null;
            }
            return this
                .fileSystems()
                .find(function (x) { return name.toLowerCase() === x.name.toLowerCase(); });
        };
        resourcesManager.prototype.getCounterStorageByName = function (name) {
            if (!name) {
                return null;
            }
            return this
                .counterStorages()
                .find(function (x) { return name.toLowerCase() === x.name.toLowerCase(); });
        };
        resourcesManager.prototype.getTimeSeriesByName = function (name) {
            if (!name) {
                return null;
            }
            return this
                .timeSeries()
                .find(function (x) { return name.toLowerCase() === x.name.toLowerCase(); });
        };
        resourcesManager.prototype.getResourceByQualifiedName = function (qualifiedName) {
            var dbPrefix = database.qualifier + "/";
            var fsPrefix = filesystem.qualifier + "/";
            //TODO: support for cs, ts
            if (qualifiedName.startsWith(dbPrefix)) {
                return this.getDatabaseByName(qualifiedName.substring(dbPrefix.length));
            }
            else if (qualifiedName.startsWith(fsPrefix)) {
                return this.getFileSystemByName(qualifiedName.substring(fsPrefix.length));
            }
            else {
                throw new Error("Unable to find resource: " + qualifiedName);
            }
        };
        resourcesManager.prototype.init = function () {
            var _this = this;
            return this.refreshResources()
                .done(function () {
                _this.activateBasedOnCurrentUrl();
                router.activate();
                _this.initialized.resolve();
            });
        };
        resourcesManager.prototype.activateAfterCreation = function (rsQualifier, resourceName) {
            this.resourceToActivate(rsQualifier + "/" + resourceName);
        };
        resourcesManager.prototype.refreshResources = function () {
            var _this = this;
            return new getResourcesCommand()
                .execute()
                .done(function (result) {
                _this.updateResources(result);
            });
            //TODO: .fail(result => this.handleRavenConnectionFailure(result))
        };
        resourcesManager.prototype.activateBasedOnCurrentUrl = function () {
            var dbUrl = appUrl.getDatabaseNameFromUrl();
            var fsUrl = appUrl.getFileSystemNameFromUrl();
            var csUrl = appUrl.getCounterStorageNameFromUrl();
            var tsUrl = appUrl.getTimeSeriesNameFromUrl();
            if (dbUrl) {
                var db = this.getDatabaseByName(dbUrl);
                this.activateIfDifferent(db, dbUrl, appUrl.forResources);
            }
            else if (fsUrl) {
                var fs = this.getFileSystemByName(fsUrl);
                this.activateIfDifferent(fs, fsUrl, appUrl.forResources);
            }
            else if (tsUrl) {
                var ts = this.getTimeSeriesByName(tsUrl);
                this.activateIfDifferent(ts, tsUrl, appUrl.forResources);
            }
            else if (csUrl) {
                var cs = this.getCounterStorageByName(csUrl);
                this.activateIfDifferent(cs, csUrl, appUrl.forResources);
            }
        };
        resourcesManager.prototype.activateIfDifferent = function (rs, rsName, urlIfNotFound) {
            var _this = this;
            this.initialized.done(function () {
                var currentActiveResource = _this.activeResourceTracker.resource();
                if (currentActiveResource != null && currentActiveResource.qualifiedName === rs.qualifiedName) {
                    return;
                }
                if (rs && !rs.disabled()) {
                    rs.activate(); //TODO: do we need this event right now?
                }
                else if (rs) {
                    messagePublisher.reportError(rs.fullTypeName + " '" + rs.name + "' is disabled!", "You can't access any section of the " + rs.fullTypeName.toLowerCase() + " while it's disabled.");
                    router.navigate(urlIfNotFound());
                }
                else {
                    messagePublisher.reportError("The resource " + rsName + " doesn't exist!");
                    router.navigate(urlIfNotFound());
                }
            });
        };
        resourcesManager.prototype.fetchStudioConfigForDatabase = function (db) {
            //TODO: fetch hot spare and studio config 
        };
        resourcesManager.prototype.activateDatabase = function (db) {
            //TODO: this.fecthStudioConfigForDatabase(db);
            this.changesContext.updateChangesApi(db, function (changes) {
                /* TODO
                changes.watchAllDocs(() => this.refreshResources()); //TODO: use cooldown - and move to footer?
                changes.watchAllIndexes(() => this.fetchDbStats(db)),
                changes.watchBulks(() => this.fetchDbStats(db))*/
                return [];
            });
        };
        resourcesManager.prototype.activateFileSystem = function (fs) {
            //TODO: ???? this.fecthStudioConfigForDatabase(new database(fs.name));
            this.changesContext.updateChangesApi(fs, function (changes) {
                //TODO: watchFsFolders("", () => this.fetchFsStats(fs))
                return [];
            });
        };
        resourcesManager.prototype.reloadDataAfterReconnection = function (rs) {
            /* TODO:
            shell.fetchStudioConfig();
            this.fetchServerBuildVersion();
            this.fetchClientBuildVersion();
            shell.fetchLicenseStatus();
            this.fetchSupportCoverage();
            this.loadServerConfig();
            this.fetchClusterTopology();
            */
            /* TODO: redirect to resources page if current resource if no longer available on list

           var activeResource = this.activeResource();
           var actualResourceObservableArray = resourceObservableArray();

           if (!!activeResource && !_.includes(actualResourceObservableArray(), activeResource)) {
               if (actualResourceObservableArray.length > 0) {
                   resourceObservableArray().first().activate();
               } else { //if (actualResourceObservableArray.length == 0)
                   shell.disconnectFromResourceChangesApi();
                   this.activeResource(null);
               }

               this.navigate(appUrl.forResources());
           }
       }*/
        };
        resourcesManager.prototype.updateResources = function (incomingData) {
            var _this = this;
            this.deleteRemovedResources(incomingData);
            incomingData.Databases.forEach(function (dbInfo) {
                _this.updateResource(dbInfo, function (name) { return _this.getDatabaseByName(name); }, database.qualifier);
            });
            /* TODO:
            incomingData.FileSystems.forEach(fsInfo => {
                this.updateResource(fsInfo, name => this.getFileSystemByName(name), filesystem.qualifier);
            });*/
        };
        resourcesManager.prototype.deleteRemovedResources = function (incomingData) {
            var existingResources = this.resources;
            var toDelete = [];
            this.databases().forEach(function (db) {
                var matchedDb = incomingData.Databases.find(function (x) { return x.Name.toLowerCase() === db.name.toLowerCase(); });
                if (!matchedDb) {
                    toDelete.push(db);
                }
            });
            /* TODO
            this.fileSystems().forEach(fs => {
                const matchedFs = incomingData.FileSystems.find(x => x.Name.toLowerCase() === fs.name.toLowerCase());
                if (!matchedFs) {
                    toDelete.push(fs);
                }
            });*/
            existingResources.removeAll(toDelete);
        };
        resourcesManager.prototype.updateResource = function (incomingResource, existingResourceFinder, resourceQualifer) {
            var matchedExistingRs = existingResourceFinder(incomingResource.Name);
            if (matchedExistingRs) {
                matchedExistingRs.updateUsing(incomingResource);
                return matchedExistingRs;
            }
            else {
                var newResource = this.createResource(resourceQualifer, incomingResource);
                var locationToInsert = _.sortedIndexBy(this.resources(), newResource, function (item) { return item.qualifiedName; });
                this.resources.splice(locationToInsert, 0, newResource);
                return newResource;
            }
        };
        resourcesManager.prototype.createResource = function (qualifer, resourceInfo) {
            if (database.qualifier === qualifer) {
                return new database(resourceInfo);
            }
            else if (filesystem.qualifier === qualifer) {
                return new filesystem(resourceInfo);
            }
            //TODO: ts, cs
            throw new Error("Unhandled resource type: " + qualifer);
        };
        resourcesManager.prototype.createGlobalNotifications = function () {
            var _this = this;
            var globalChanges = changesContext.default.globalChangesApi();
            return [
                globalChanges.watchItemsStartingWith("db/", function (e) { return _this.onResourceUpdateReceivedViaChangesApi(e); }),
            ];
        };
        resourcesManager.prototype.onResourceUpdateReceivedViaChangesApi = function (event) {
            var _this = this;
            var resource = this.getResourceByQualifiedName(event.Id);
            if (event.Operation === "Delete" && resource) {
                changesContext.default.disconnectIfCurrent(resource, "ResourceDeleted");
                this.resources.remove(resource);
            }
            else if (event.Operation === "Write") {
                var _a = event.Id.split("/", 2), prefix_1 = _a[0], name_1 = _a[1];
                new getResourceCommand(prefix_1, name_1)
                    .execute()
                    .done(function (rsInfo) {
                    if (rsInfo.Disabled) {
                        changesContext.default.disconnectIfCurrent(resource, "ResourceDisabled");
                    }
                    //TODO: it supports db only for now!
                    if (prefix_1 !== database.qualifier) {
                        throw new Error("we support db only for now!"); //TODO: delete me once we support more types
                    }
                    var updatedResource = _this.updateResource(rsInfo, function (name) { return _this.getDatabaseByName(name); }, database.qualifier);
                    var toActivate = _this.resourceToActivate();
                    if (toActivate && toActivate === event.Id) {
                        updatedResource.activate();
                        _this.resourceToActivate(null);
                    }
                });
            }
        };
        return resourcesManager;
    }());
    resourcesManager.default = new resourcesManager();
    return resourcesManager;
});

//# sourceMappingURL=resourcesManager.js.map
