/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports"], function (require, exports) {
    "use strict";
    var periodicExportSetup = (function () {
        function periodicExportSetup() {
            var _this = this;
            this.onDiskExportEnabled = ko.observable(true);
            this.remoteUploadEnabled = ko.observable(false);
            this.localFolderName = ko.observable();
            this.unsupported = ko.observable(false);
            this.active = ko.observable(false);
            this.type = ko.observable();
            this.mainValue = ko.observable();
            this.azureRemoteFolderName = ko.observable();
            this.s3RemoteFolderName = ko.observable();
            this.awsAccessKey = ko.observable();
            this.awsSecretKey = ko.observable();
            this.awsRegionName = ko.observable();
            this.awsSecretKeyDecryptionFailed = ko.observable(false);
            this.azureStorageKeyDecryptionFailed = ko.observable(false);
            this.azureStorageAccount = ko.observable();
            this.azureStorageKey = ko.observable();
            this.incrementalBackupInterval = ko.observable();
            this.incrementalBackupIntervalUnit = ko.observable();
            this.fullBackupInterval = ko.observable();
            this.fullBackupIntervalUnit = ko.observable();
            this.FILE_SYSTEM = "fileSystem";
            this.GLACIER_VAULT = "glacierVault";
            this.S3_BUCKET = "s3bucket";
            this.AZURE_STORAGE = "azureStorage";
            this.TU_MINUTES = "minutes";
            this.TU_HOURS = "hours";
            this.TU_DAYS = "days";
            this.availablePeriodicExports = [
                { label: "Glacier Vault Name:", value: this.GLACIER_VAULT },
                { label: "S3 Bucket Name:", value: this.S3_BUCKET },
                { label: "Azure Storage Container:", value: this.AZURE_STORAGE }
            ];
            this.availableAwsRegionEndpoints = [
                { label: "US East (Virginia)", value: "us-east-1" },
                { label: "US East (Ohio)", value: "us-east-2" },
                { label: "US West (N. California)", value: "us-west-1" },
                { label: "US West (Oregon)", value: "us-west-2" },
                { label: "EU (Ireland)", value: "eu-west-1" },
                { label: "EU (Frankfurt)", value: "eu-central-1" },
                { label: "Asia Pacific (Tokyo)", value: "ap-northeast-1" },
                { label: "Asia Pacific (Seoul)", value: "ap-northeast-2" },
                { label: "Asia Pacific (Singapore)", value: "ap-southeast-1" },
                { label: "Asia Pacific (Sydney)", value: "ap-southeast-2" },
                { label: "Asia Pacific (Mumbai)", value: "ap-south-1" },
                { label: "South America (São Paulo)", value: "sa-east-1" }
            ];
            this.availableIntervalUnits = [this.TU_MINUTES, this.TU_HOURS, this.TU_DAYS];
            this.mainPlaceholder = ko.computed(function () {
                switch (_this.type()) {
                    case _this.GLACIER_VAULT:
                        return "vault name only e.g. myvault";
                    case _this.S3_BUCKET:
                        return "bucket name only e.g. mybucket";
                    case _this.AZURE_STORAGE:
                        return "container name only e.g. mycontainer";
                }
            }, this);
            this.additionalAwsInfoRequired = ko.computed(function () {
                var type = _this.type();
                return _this.remoteUploadEnabled() && jQuery.inArray(type, [_this.GLACIER_VAULT, _this.S3_BUCKET]) !== -1;
            }, this);
            this.isGlaceirVault = ko.computed(function () { return _this.remoteUploadEnabled() && _this.type() === _this.GLACIER_VAULT; });
            this.isS3Bucket = ko.computed(function () { return _this.remoteUploadEnabled() && _this.type() === _this.S3_BUCKET; });
            this.additionalAzureInfoRequired = ko.computed(function () {
                var type = _this.type();
                return _this.remoteUploadEnabled() && type === _this.AZURE_STORAGE;
            }, this);
            this.mainValueCustomValidity = ko.computed(function () {
                var mainValue = _this.mainValue();
                switch (_this.type()) {
                    case _this.GLACIER_VAULT:
                        return _this.validateGlacierVaultName(mainValue);
                    case _this.S3_BUCKET:
                        return _this.validateS3Bucket(mainValue);
                    case _this.AZURE_STORAGE:
                        return _this.validateAzureContainerNAme(mainValue);
                }
                return "";
            });
        }
        /*
        Names can be between 1 and 255 characters long.
    
        Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (hyphen), and '.' (period).
        */
        periodicExportSetup.prototype.validateGlacierVaultName = function (vaultName) {
            if (vaultName == null || vaultName.length < 1 || vaultName.length > 255) {
                return "Vault name must be at least 1 and no more than 255 characters long.";
            }
            var regEx = /^[A-Za-z0-9_\.-]+$/;
            if (!regEx.test(vaultName)) {
                return "Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (hyphen), and '.' (period).";
            }
            return "";
        };
        /*
        Bucket names must :
            - be at least 3 and no more than 63 characters long.
            - be a series of one or more labels.
                Adjacent labels are separated by a single period (.).
                Bucket names can contain lowercase letters, numbers, and hyphens.
                Each label must start and end with a lowercase letter or a number.
            - not be formatted as an IP address (e.g., 192.168.5.4).
        */
        periodicExportSetup.prototype.validateS3Bucket = function (bucketName) {
            if (bucketName == null || bucketName.length < 3 || bucketName.length > 63) {
                return "Bucket name must be at least 3 and no more than 63 characters long";
            }
            if (bucketName[0] == '.') {
                return 'Bucket name cannot start with a period (.)';
            }
            if (bucketName[bucketName.length - 1] == '.') {
                return 'Bucket name cannot end with a period (.)';
            }
            if (bucketName.indexOf("..") > -1) {
                return 'There can be only one period between labels';
            }
            var labels = bucketName.split(".");
            var labelRegExp = /^[a-z0-9-]+$/;
            var validLabel = function (label) {
                if (label == null || label.length == 0) {
                    return false;
                }
                if (labelRegExp.test(label) == false) {
                    return false;
                }
                if (label[0] == "-" || label[label.length - 1] == "-") {
                    return false;
                }
                return true;
            };
            if (labels.some(function (l) { return !validLabel(l); })) {
                return "Bucket name is invalid";
            }
            var ipRegExp = /^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/;
            if (ipRegExp.test(bucketName)) {
                return "Bucket name must not be formatted as an IP address (e.g., 192.168.5.4).";
            }
            return "";
        };
        /*
        Container names must:
            - start with a letter or number, and can contain only letters, numbers, and the dash (-) character.
            - every dash (-) character must be immediately preceded and followed by a letter or number;
            - consecutive dashes are not permitted in container names.
            - all letters in a container name must be lowercase.
            - container names must be from 3 through 63 characters long.
        */
        periodicExportSetup.prototype.validateAzureContainerNAme = function (containerName) {
            if (containerName == null || containerName.length < 3 || containerName.length > 63) {
                return "Container name must be at least 3 and no more than 63 characters long";
            }
            var regExp = /^[a-z0-9-]+$/;
            if (!regExp.test(containerName)) {
                return "Allowed characters are a-z,0-9 and '-' (hyphen).";
            }
            if (containerName[0] == "-" || containerName[containerName.length - 1] == "-") {
                return "Container name must start and end with a letter or number";
            }
            var twoDashes = /--/;
            if (twoDashes.test(containerName)) {
                return "Consecutive dashes are not permitted in container names";
            }
            return "";
        };
        periodicExportSetup.prototype.fromDto = function (dto) {
            if (dto) {
                this.awsRegionName(dto.AwsRegionName);
                this.setupTypeAndMainValue(dto);
                this.s3RemoteFolderName(dto.S3RemoteFolderName);
                this.azureRemoteFolderName(dto.AzureRemoteFolderName);
                var incr = this.prepareBackupInterval(dto.IntervalMilliseconds);
                this.incrementalBackupInterval(incr[0]);
                this.incrementalBackupIntervalUnit(incr[1]);
                var full = this.prepareBackupInterval(dto.FullExportIntervalMilliseconds);
                this.fullBackupInterval(full[0]);
                this.fullBackupIntervalUnit(full[1]);
                this.active(dto.Active);
            }
        };
        periodicExportSetup.prototype.toDto = function () {
            return {
                Active: this.active(),
                GlacierVaultName: this.prepareMainValue(this.GLACIER_VAULT),
                S3BucketName: this.prepareMainValue(this.S3_BUCKET),
                AwsRegionName: this.awsRegionName(),
                AzureStorageContainer: this.prepareMainValue(this.AZURE_STORAGE),
                LocalFolderName: this.onDiskExportEnabled() ? this.localFolderName() : null,
                S3RemoteFolderName: this.isS3Bucket() ? this.s3RemoteFolderName() : null,
                AzureRemoteFolderName: this.additionalAzureInfoRequired() ? this.azureRemoteFolderName() : null,
                IntervalMilliseconds: this.convertToMilliseconds(this.incrementalBackupInterval(), this.incrementalBackupIntervalUnit()),
                FullExportIntervalMilliseconds: this.convertToMilliseconds(this.fullBackupInterval(), this.fullBackupIntervalUnit())
            };
        };
        periodicExportSetup.prototype.prepareMainValue = function (expectedType) {
            return ((this.type() === expectedType && this.remoteUploadEnabled()) ? this.mainValue() : null);
        };
        periodicExportSetup.prototype.convertToMilliseconds = function (value, unit) {
            if (value && unit) {
                switch (unit) {
                    case this.TU_MINUTES:
                        return value * 1000 * 60;
                    case this.TU_HOURS:
                        return value * 1000 * 60 * 60;
                    case this.TU_DAYS:
                        return value * 1000 * 60 * 60 * 24;
                }
            }
            return null;
        };
        periodicExportSetup.prototype.fromDatabaseSettingsDto = function (dbSettingsDto) {
            this.dbSettingsDto = dbSettingsDto;
            this.awsAccessKey(dbSettingsDto["Settings"]["Raven/AWSAccessKey"]);
            this.awsSecretKey(dbSettingsDto["SecuredSettings"]["Raven/AWSSecretKey"]);
            this.azureStorageAccount(dbSettingsDto["Settings"]["Raven/AzureStorageAccount"]);
            this.azureStorageKey(dbSettingsDto["SecuredSettings"]["Raven/AzureStorageKey"]);
            if (periodicExportSetup.decryptFailedValue === this.awsSecretKey()) {
                this.awsSecretKey("");
                this.awsSecretKeyDecryptionFailed(true);
            }
            if (periodicExportSetup.decryptFailedValue === this.azureStorageKey()) {
                this.azureStorageKey("");
                this.azureStorageKeyDecryptionFailed(true);
            }
        };
        periodicExportSetup.prototype.toDatabaseSettingsDto = function () {
            if (this.additionalAwsInfoRequired()) {
                this.dbSettingsDto["Settings"]["Raven/AWSAccessKey"] = this.awsAccessKey();
                this.dbSettingsDto["SecuredSettings"]["Raven/AWSSecretKey"] = this.awsSecretKey();
            }
            else {
                delete this.dbSettingsDto["Settings"]["Raven/AWSAccessKey"];
                delete this.dbSettingsDto["SecuredSettings"]["Raven/AWSSecretKey"];
            }
            if (this.additionalAzureInfoRequired()) {
                this.dbSettingsDto["Settings"]["Raven/AzureStorageAccount"] = this.azureStorageAccount();
                this.dbSettingsDto["SecuredSettings"]["Raven/AzureStorageKey"] = this.azureStorageKey();
            }
            else {
                delete this.dbSettingsDto["Settings"]["Raven/AzureStorageAccount"];
                delete this.dbSettingsDto["SecuredSettings"]["Raven/AzureStorageKey"];
            }
            return this.dbSettingsDto;
        };
        periodicExportSetup.prototype.removeDatabaseSettings = function () {
            delete this.dbSettingsDto["Settings"]["Raven/AWSAccessKey"];
            delete this.dbSettingsDto["SecuredSettings"]["Raven/AWSSecretKey"];
            delete this.dbSettingsDto["Settings"]["Raven/AzureStorageAccount"];
            delete this.dbSettingsDto["SecuredSettings"]["Raven/AzureStorageKey"];
            return this.dbSettingsDto;
        };
        periodicExportSetup.prototype.getEtag = function () {
            return this.toDatabaseSettingsDto()["@metadata"]["@etag"];
        };
        periodicExportSetup.prototype.setEtag = function (newEtag) {
            this.toDatabaseSettingsDto()["@metadata"]["@etag"] = newEtag;
        };
        periodicExportSetup.prototype.resetDecryptionFailures = function () {
            this.awsSecretKeyDecryptionFailed(false);
            this.azureStorageKeyDecryptionFailed(false);
        };
        periodicExportSetup.prototype.setupTypeAndMainValue = function (dto) {
            var count = 0;
            if (dto.LocalFolderName) {
                this.localFolderName(dto.LocalFolderName);
                this.onDiskExportEnabled(true);
            }
            if (dto.GlacierVaultName) {
                count += 1;
                this.type(this.GLACIER_VAULT);
                this.mainValue(dto.GlacierVaultName);
            }
            if (dto.S3BucketName) {
                count += 1;
                this.type(this.S3_BUCKET);
                this.mainValue(dto.S3BucketName);
            }
            if (dto.AzureStorageContainer) {
                count += 1;
                this.type(this.AZURE_STORAGE);
                this.mainValue(dto.AzureStorageContainer);
            }
            if (count > 0) {
                this.remoteUploadEnabled(true);
            }
            this.unsupported(count > 1);
        };
        periodicExportSetup.prototype.prepareBackupInterval = function (milliseconds) {
            if (milliseconds) {
                var seconds = milliseconds / 1000;
                var minutes = seconds / 60;
                var hours = minutes / 60;
                if (this.isValidTimeValue(hours)) {
                    var days = hours / 24;
                    if (this.isValidTimeValue(days)) {
                        return [days, this.TU_DAYS];
                    }
                    return [hours, this.TU_HOURS];
                }
                return [minutes, this.TU_MINUTES];
            }
            return [0, this.TU_MINUTES];
        };
        periodicExportSetup.prototype.isValidTimeValue = function (value) {
            return value >= 1 && value % 1 === 0;
        };
        periodicExportSetup.prototype.copyFrom = function (from) {
            this.onDiskExportEnabled(from.onDiskExportEnabled());
            this.remoteUploadEnabled(from.remoteUploadEnabled());
            this.localFolderName(from.localFolderName());
            this.unsupported(from.unsupported());
            this.active(from.active());
            this.type(from.type());
            this.mainValue(from.mainValue());
            this.awsAccessKey(from.awsAccessKey());
            this.awsSecretKey(from.awsSecretKey());
            this.awsRegionName(from.awsRegionName());
            this.s3RemoteFolderName(from.s3RemoteFolderName());
            this.azureStorageAccount(from.azureStorageAccount());
            this.azureStorageKey(from.azureStorageKey());
            this.azureRemoteFolderName(from.azureRemoteFolderName());
            this.incrementalBackupInterval(from.incrementalBackupInterval());
            this.incrementalBackupIntervalUnit(from.incrementalBackupIntervalUnit());
            this.fullBackupInterval(from.fullBackupInterval());
            this.fullBackupIntervalUnit(from.fullBackupIntervalUnit());
        };
        return periodicExportSetup;
    }());
    periodicExportSetup.decryptFailedValue = "<data could not be decrypted>";
    return periodicExportSetup;
});

//# sourceMappingURL=periodicExportSetup.js.map
