/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports", "models/database/index/luceneField", "models/database/index/spatialIndexField"], function (require, exports, luceneField, spatialIndexField) {
    "use strict";
    var indexDefinition = (function () {
        function indexDefinition(dto) {
            var _this = this;
            this.name = ko.observable();
            this.map = ko.observable();
            this.maps = ko.observableArray();
            this.reduce = ko.observable();
            this.luceneFields = ko.observableArray();
            this.isTestIndex = ko.observable(false);
            this.isSideBySideIndex = ko.observable(false);
            this.numOfLuceneFields = ko.computed(function () { return _this.luceneFields().length; });
            // This is an amalgamation of several properties from the index (Fields, Stores, Indexes, SortOptions, Analyzers, Suggestions, TermVectors) 
            // Stored as multiple luceneFields for the sake of data binding.
            // Each luceneField corresponds to a Field box in the index editor UI.
            this.spatialFields = ko.observableArray();
            this.numOfSpatialFields = ko.computed(function () { return _this.spatialFields().length; });
            this.maxIndexOutputsPerDocument = ko.observable(0);
            this.storeAllFields = ko.observable(false);
            this.fields = ko.observableArray();
            //TODO: this.analyzers = dto.Analyzers;
            //TODO: this.fields(dto.Fields);
            //TODO: this.indexes = dto.Indexes;
            //TODO: this.internalFieldsMapping = dto.InternalFieldsMapping;
            this.isTestIndex(dto.IsTestIndex);
            this.isSideBySideIndex(dto.IsSideBySideIndex);
            //TODO: this.isCompiled = dto.IsCompiled;
            //TODO: this.isMapReduce = dto.IsMapReduce;
            this.lockMode = dto.LockMode;
            //TODO: this.map(dto.Map);
            this.maps(dto.Maps.map(function (m) { return ko.observable(m); }));
            this.name(dto.Name);
            this.reduce(dto.Reduce);
            //TODO: this.sortOptions = dto.SortOptions;
            //TODO: this.spatialIndexes = dto.SpatialIndexes;
            //TODO: this.stores = dto.Stores;
            //TODO: this.suggestionsOptions = dto.SuggestionsOptions;
            //TODO: this.termVectors = dto.TermVectors;
            this.type = dto.Type;
            this.luceneFields(this.parseFields());
            this.spatialFields(this.parseSpatialFields());
            this.maxIndexOutputsPerDocument(0);
            //TODO this.maxIndexOutputsPerDocument(dto.MaxIndexOutputsPerDocument ? dto.MaxIndexOutputsPerDocument : 0);
            this.storeAllFields(this.isStoreAllFields());
        }
        indexDefinition.prototype.isStoreAllFields = function () {
            if (this.stores && this.stores.hasOwnProperty("__all_fields")) {
                return this.stores["__all_fields"] === "Yes";
            }
            return false;
        };
        indexDefinition.prototype.setOrRemoveStoreAllFields = function (add) {
            if (add) {
                this.stores["__all_fields"] = "Yes";
            }
            else {
                delete this.stores["__all_fields"];
            }
            this.storeAllFields(this.isStoreAllFields());
        };
        indexDefinition.prototype.setStoreAllFieldsToObject = function (obj) {
            if (this.isStoreAllFields())
                obj["__all_fields"] = "Yes";
            return obj;
        };
        indexDefinition.prototype.toDto = function () {
            return {
                //Analyzers: this.makeFieldObject(f => f.indexing() === "Analyzed", f => f.analyzer()),
                Fields: {},
                //Indexes: this.makeFieldObject(f => f.indexing() !== "Default", f => f.indexing()),
                //InternalFieldsMapping: this.internalFieldsMapping,
                IsTestIndex: this.isTestIndex(),
                IsSideBySideIndex: this.isSideBySideIndex(),
                //IsCompiled: this.isCompiled,
                //IsMapReduce: this.isMapReduce,
                LockMode: "Unlock",
                IndexId: null,
                //Map: this.maps()[0](),
                Maps: this.maps().map(function (m) { return m(); }).filter(function (m) { return m && m.length > 0; }),
                Name: this.name(),
                Reduce: this.reduce(),
                //SortOptions: this.makeFieldObject(f => f.sort() !== "None", f => f.sort()),
                //SpatialIndexes: this.makeSpatialIndexesObject(),
                //Stores: this.setStoreAllFieldsToObject(this.makeFieldObject(f => f.stores() === "Yes", f => f.stores())),
                //SuggestionsOptions: this.luceneFields().filter(x => x.suggestionEnabled()).map(x => x.name()),
                //TermVectors: this.makeFieldObject(f => f.termVector() !== "No", f => f.termVector()),
                Type: this.type,
                Configuration: null //TODO
            };
        };
        indexDefinition.empty = function () {
            return new indexDefinition({
                Fields: {},
                IndexId: null,
                Maps: [""],
                Name: "",
                LockMode: "Unlock",
                Reduce: "",
                Configuration: null,
                //TODO IndexVersion: -1,
                IsSideBySideIndex: false,
                IsTestIndex: false,
                //TODO MaxIndexOutputsPerDocument: null,
                Type: "Map"
            });
        };
        indexDefinition.prototype.makeSpatialIndexesObject = function () {
            var spatialIndexesObj = {};
            this.spatialFields().forEach(function (f) { return spatialIndexesObj[f.name()] = f.toDto(); });
            return spatialIndexesObj;
        };
        indexDefinition.prototype.makeFieldObject = function (filter, selector) {
            var obj = {};
            this.luceneFields()
                .filter(filter)
                .forEach(function (f) { return obj[f.name()] = selector(f); });
            return obj;
        };
        indexDefinition.prototype.parseFields = function () {
            var _this = this;
            var fieldSources = [
                this.analyzers,
                this.indexes,
                this.sortOptions,
                this.stores,
                this.suggestionsOptions,
                this.termVectors
            ];
            var keys = [];
            for (var i = 0; i < fieldSources.length; i++) {
                var src = fieldSources[i];
                if (src == null)
                    continue;
                for (var key in src) {
                    if (src.hasOwnProperty(key) === true && !_.includes(keys, key))
                        keys.push(key);
                }
            }
            return keys
                .map(function (fieldName) {
                var suggestionsEnabled = _this.suggestionsOptions && _this.suggestionsOptions.indexOf(fieldName) >= 0;
                return new luceneField(fieldName, _this.stores[fieldName], _this.indexes[fieldName], _this.sortOptions[fieldName], _this.analyzers[fieldName], suggestionsEnabled, _this.termVectors[fieldName], _this.fields());
            });
        };
        indexDefinition.prototype.parseSpatialFields = function () {
            // The spatial fields are stored as properties on the .spatialIndexes object.
            // The property names will be one of the .fields.
            var fields = [];
            if (this.spatialIndexes == null)
                return fields;
            for (var key in this.spatialIndexes) {
                var spatialIndex = this.spatialIndexes[key];
                fields.push(new spatialIndexField(key, spatialIndex));
            }
            return fields;
        };
        return indexDefinition;
    }());
    return indexDefinition;
});

//# sourceMappingURL=indexDefinition.js.map
