define(["require", "exports", "models/database/replication/replicationPatchScript"], function (require, exports, replicationPatchScript) {
    "use strict";
    var replicationDestination = (function () {
        function replicationDestination(dto) {
            var _this = this;
            this.url = ko.observable();
            this.username = ko.observable();
            this.password = ko.observable();
            this.domain = ko.observable();
            this.apiKey = ko.observable();
            this.database = ko.observable();
            this.transitiveReplicationBehavior = ko.observable();
            this.ignoredClient = ko.observable();
            this.disabled = ko.observable();
            this.clientVisibleUrl = ko.observable();
            this.skipIndexReplication = ko.observable();
            this.specifiedCollections = ko.observableArray().extend({ required: false });
            this.withScripts = ko.observableArray([]);
            this.enableReplicateOnlyFromCollections = ko.observable();
            this.name = ko.computed(function () {
                var prefix = _this.disabled() ? "[disabled]" : null;
                var database = _this.database();
                var on = _this.database() && _this.url() ? "on" : null;
                var url = _this.url();
                return [prefix, database, on, url]
                    .filter(function (s) { return !!s; })
                    .join(" ")
                    || "[empty]";
            });
            this.isValid = ko.computed(function () { return _this.url() != null && _this.url().length > 0; });
            // data members for the ui
            this.isUserCredentials = ko.observable(false);
            this.isApiKeyCredentials = ko.observable(false);
            this.credentialsType = ko.computed(function () {
                if (_this.isUserCredentials()) {
                    return "user";
                }
                else if (_this.isApiKeyCredentials()) {
                    return "api-key";
                }
                else {
                    return "none";
                }
            });
            this.url(dto.Url);
            this.username(dto.Username);
            this.password(dto.Password);
            this.domain(dto.Domain);
            this.apiKey(dto.ApiKey);
            this.database(dto.Database);
            this.transitiveReplicationBehavior(dto.TransitiveReplicationBehavior);
            this.ignoredClient(dto.IgnoredClient);
            this.disabled(dto.Disabled);
            this.clientVisibleUrl(dto.ClientVisibleUrl);
            this.skipIndexReplication(dto.SkipIndexReplication);
            this.specifiedCollections(this.mapSpecifiedCollections(dto.SpecifiedCollections));
            this.withScripts(this.specifiedCollections().filter(function (x) { return typeof (x.script()) !== "undefined"; }).map(function (x) { return x.collection(); }));
            this.enableReplicateOnlyFromCollections = ko.observable(this.specifiedCollections().length > 0);
            if (this.username()) {
                this.isUserCredentials(true);
            }
            else if (this.apiKey()) {
                this.isApiKeyCredentials(true);
            }
            this.skipIndexReplication.subscribe(function () { return ko.postbox.publish('skip-index-replication'); });
        }
        replicationDestination.prototype.clearApiKeyCredentials = function () {
            this.apiKey(null);
        };
        replicationDestination.prototype.clearUserCredentials = function () {
            this.username(null);
            this.password(null);
            this.domain(null);
        };
        replicationDestination.prototype.useUserCredentials = function () {
            this.isUserCredentials(true);
            this.isApiKeyCredentials(false);
            this.clearApiKeyCredentials();
        };
        replicationDestination.prototype.useApiKeyCredentials = function () {
            this.isApiKeyCredentials(true);
            this.isUserCredentials(false);
            this.clearUserCredentials();
        };
        replicationDestination.prototype.useNoCredentials = function () {
            this.isUserCredentials(false);
            this.isApiKeyCredentials(false);
            this.clearUserCredentials();
            this.clearApiKeyCredentials();
        };
        replicationDestination.prototype.toggleIsAdvancedShows = function (item, event) {
            $(event.target).next().toggle();
        };
        replicationDestination.prototype.mapSpecifiedCollections = function (input) {
            var result = [];
            if (!input) {
                return result;
            }
            for (var key in input) {
                if (input.hasOwnProperty(key)) {
                    var item = replicationPatchScript.empty();
                    item.collection(key);
                    var script = input[key];
                    if (script === null) {
                        script = undefined;
                    }
                    item.script(script);
                    result.push(item);
                }
            }
            return result;
        };
        replicationDestination.prototype.isSelected = function (col) {
            var cols = this.specifiedCollections();
            return !!cols.find(function (c) { return c.collection() === col.name; });
        };
        replicationDestination.prototype.hasScript = function (col) {
            var cols = this.specifiedCollections();
            var item = cols.find(function (c) { return c.collection() === col.name; });
            if (!item) {
                return false;
            }
            return typeof (item.script()) !== "undefined";
        };
        replicationDestination.prototype.toggleSkipIndexReplication = function () {
            this.skipIndexReplication.toggle();
        };
        replicationDestination.empty = function (databaseName) {
            return new replicationDestination({
                Url: location.protocol + "//" + location.host,
                Username: null,
                Password: null,
                Domain: null,
                ApiKey: null,
                Database: databaseName,
                TransitiveReplicationBehavior: "Replicate",
                IgnoredClient: false,
                Disabled: false,
                ClientVisibleUrl: null,
                SkipIndexReplication: false,
                SpecifiedCollections: {}
            });
        };
        replicationDestination.prototype.enable = function () {
            this.disabled(false);
        };
        replicationDestination.prototype.disable = function () {
            this.disabled(true);
        };
        replicationDestination.prototype.includeFailover = function () {
            this.ignoredClient(false);
        };
        replicationDestination.prototype.skipFailover = function () {
            this.ignoredClient(true);
        };
        replicationDestination.prototype.toDto = function () {
            return {
                Url: this.prepareUrl(),
                Username: this.username(),
                Password: this.password(),
                Domain: this.domain(),
                ApiKey: this.apiKey(),
                Database: this.database(),
                TransitiveReplicationBehavior: this.transitiveReplicationBehavior(),
                IgnoredClient: this.ignoredClient(),
                Disabled: this.disabled(),
                ClientVisibleUrl: this.clientVisibleUrl(),
                SkipIndexReplication: this.skipIndexReplication(),
                SpecifiedCollections: this.enableReplicateOnlyFromCollections() ? this.specifiedCollectionsToMap() : null
            };
        };
        replicationDestination.prototype.specifiedCollectionsToMap = function () {
            var result = {};
            var collections = this.specifiedCollections();
            for (var i = 0; i < collections.length; i++) {
                var item = collections[i];
                var script = item.script();
                if (!script) {
                    script = null;
                }
                result[item.collection()] = script;
            }
            return result;
        };
        replicationDestination.prototype.prepareUrl = function () {
            var url = this.url();
            if (url && url.charAt(url.length - 1) === "/") {
                url = url.substring(0, url.length - 1);
            }
            return url;
        };
        replicationDestination.prototype.addNewCollection = function () {
            this.specifiedCollections.push(replicationPatchScript.empty());
        };
        replicationDestination.prototype.removeCollection = function (item) {
            this.specifiedCollections.remove(item);
        };
        return replicationDestination;
    }());
    return replicationDestination;
});

//# sourceMappingURL=replicationDestination.js.map
