/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports", "configuration"], function (require, exports, configuration) {
    "use strict";
    var resourceCreationModel = (function () {
        function resourceCreationModel() {
            this.name = ko.observable("");
            this.activeBundles = ko.observableArray([]);
            this.dataPath = ko.observable();
            this.journalsPath = ko.observable();
            this.tempPath = ko.observable();
            this.encryption = {
                key: ko.observable(),
                algorithm: ko.observable("Rijndael"),
                keyBitsPreference: ko.observable(256),
                encryptIndexes: ko.observable(true)
            };
            this.encryptionAlgorithmsOptions = ["DES", "RC2", "Rijndael", "Triple DES"];
            this.encryptionKeyBitsOptions = [128, 192, 256];
            this.globalValidationGroup = ko.validatedObservable({
                name: this.name
            });
            this.encryptionValidationGroup = ko.validatedObservable({
                key: this.encryption.key
            });
        }
        resourceCreationModel.prototype.setupValidation = function (resourceDoesntExist) {
            var rg1 = /^[^\\/:\*\?"<>\|]*$/; // forbidden characters \ / : * ? " < > |
            var rg3 = /^(nul|prn|con|lpt[0-9]|com[0-9])(\.|$)/i; // forbidden file names
            this.setupPathValidation(this.dataPath, "Data");
            this.setupPathValidation(this.tempPath, "Temp");
            this.setupPathValidation(this.journalsPath, "Journals");
            this.name.extend({
                required: true,
                maxLength: 230,
                validation: [
                    {
                        validator: resourceDoesntExist,
                        message: _.upperFirst(this.resourceType) + " already exists"
                    }, {
                        validator: function (val) { return rg1.test(val); },
                        message: "The {0} name can't contain any of the following characters: \\ / : * ? \" < > |",
                        params: this.resourceType
                    }, {
                        validator: function (val) { return !val.startsWith("."); },
                        message: "The " + this.resourceType + " name can't start with a dot!"
                    }, {
                        validator: function (val) { return !val.endsWith("."); },
                        message: "The " + this.resourceType + " name can't end with a dot!"
                    }, {
                        validator: function (val) { return !rg3.test(val); },
                        message: "The name {0} is forbidden for use!",
                        params: this.name
                    }
                ]
            });
            this.encryption.key.extend({
                required: true,
                base64: true
            });
        };
        resourceCreationModel.prototype.setupPathValidation = function (observable, name) {
            var maxLength = 248;
            var rg1 = /^[^*?"<>\|]*$/; // forbidden characters * ? " < > |
            var rg3 = /^(nul|prn|con|lpt[0-9]|com[0-9])(\.|$)/i; // forbidden file names
            observable.extend({
                maxLength: {
                    params: maxLength,
                    message: "Path name for '" + name + "' can't exceed " + maxLength + " characters!"
                },
                validation: [{
                        validator: function (val) { return rg1.test(val); },
                        message: "{0} path can't contain any of the following characters: * ? \" < > |",
                        params: name
                    },
                    {
                        validator: function (val) { return !rg3.test(val); },
                        message: "The name {0} is forbidden for use!",
                        params: this.name
                    }]
            });
        };
        resourceCreationModel.prototype.fillEncryptionSettingsIfNeeded = function (securedSettings) {
            if (_.includes(this.activeBundles(), "Encryption")) {
                securedSettings[configuration.encryption.encryptionKey] = this.encryption.key();
                securedSettings[configuration.encryption.algorithmType] = this.getEncryptionAlgorithmFullName(this.encryption.algorithm());
                securedSettings[configuration.encryption.encryptionKeyBitsPreference] = this.encryption.keyBitsPreference().toString();
                securedSettings[configuration.encryption.encryptIndexes] = this.encryption.encryptIndexes();
            }
        };
        resourceCreationModel.prototype.getEncryptionAlgorithmFullName = function (encrytion) {
            switch (encrytion) {
                case "DES":
                    return "System.Security.Cryptography.DESCryptoServiceProvider, mscorlib";
                case "RC2":
                    return "System.Security.Cryptography.RC2CryptoServiceProvider, mscorlib";
                case "Rijndael":
                    return "System.Security.Cryptography.RijndaelManaged, mscorlib";
                default:
                    return "System.Security.Cryptography.TripleDESCryptoServiceProvider, mscorlib";
            }
        };
        resourceCreationModel.prototype.setEncryptionAlgorithm = function (value) {
            this.encryption.algorithm(value);
        };
        resourceCreationModel.prototype.setEncryptionBits = function (value) {
            this.encryption.keyBitsPreference(value);
        };
        resourceCreationModel.prototype.setIndexEncryption = function (value) {
            this.encryption.encryptIndexes(value);
        };
        return resourceCreationModel;
    }());
    return resourceCreationModel;
});

//# sourceMappingURL=resourceCreationModel.js.map
