/// <reference path="../../../../typings/tsd.d.ts"/>
define(["require", "exports", "common/generalUtils", "common/shell/activeResourceTracker"], function (require, exports, generalUtils, activeResourceTracker) {
    "use strict";
    var resourceInfo = (function () {
        function resourceInfo(dto) {
            this.bundles = ko.observableArray();
            this.isAdmin = ko.observable();
            this.disabled = ko.observable();
            this.totalSize = ko.observable();
            this.errors = ko.observable();
            this.alerts = ko.observable();
            this.licensed = ko.observable(true); //TODO: bind this value
            this.uptime = ko.observable();
            this.lastFullOrIncrementalBackup = ko.observable();
            this.backupStatus = ko.observable();
            this.backupEnabled = ko.observable();
            this.filteredOut = ko.observable(false);
            this.initializeObservables();
        }
        resourceInfo.extractQualifierAndNameFromNotification = function (input) {
            return { qualifier: input.substr(0, 2), name: input.substr(3) };
        };
        Object.defineProperty(resourceInfo.prototype, "qualifiedName", {
            get: function () {
                return this.qualifier + "/" + this.name;
            },
            enumerable: true,
            configurable: true
        });
        resourceInfo.findLastBackupDate = function (dto) {
            var lastFull = dto.LastFullBackup;
            var lastIncrementalBackup = dto.LastIncrementalBackup;
            if (lastFull && lastIncrementalBackup) {
                return lastFull > lastIncrementalBackup ? lastFull : lastIncrementalBackup;
            }
            else if (lastFull) {
                return lastFull;
            }
            return lastIncrementalBackup;
        };
        resourceInfo.prototype.update = function (dto) {
            this.name = dto.Name;
            this.disabled(dto.Disabled);
            this.isAdmin(dto.IsAdmin);
            this.totalSize(dto.TotalSize ? dto.TotalSize.HumaneSize : null);
            this.errors(dto.Errors);
            this.alerts(dto.Alerts);
            this.bundles(dto.Bundles);
            this.uptime(generalUtils.timeSpanAsAgo(dto.UpTime, false));
            this.backupEnabled(!!dto.BackupInfo);
            if (this.backupEnabled()) {
                var lastBackup = resourceInfo.findLastBackupDate(dto.BackupInfo);
                this.lastFullOrIncrementalBackup(moment(new Date(lastBackup)).fromNow());
                this.backupStatus(this.computeBackupStatus(dto.BackupInfo));
            }
        };
        resourceInfo.prototype.computeBackupStatus = function (dto) {
            if (!dto.LastFullBackup && !dto.LastIncrementalBackup) {
                return "text-danger";
            }
            var fullBackupInterval = moment.duration(dto.FullBackupInterval).asSeconds();
            var incrementalBackupInterval = moment.duration(dto.IncrementalBackupInterval).asSeconds();
            var interval = (incrementalBackupInterval === 0) ? fullBackupInterval : Math.min(incrementalBackupInterval, fullBackupInterval);
            var lastBackup = new Date(resourceInfo.findLastBackupDate(dto));
            var secondsSinceLastBackup = moment.duration(moment().diff(moment(lastBackup))).asSeconds();
            return (interval * 1.2 < secondsSinceLastBackup) ? "text-warning" : "text-success";
        };
        resourceInfo.prototype.initializeObservables = function () {
            var _this = this;
            this.online = ko.pureComputed(function () {
                return !!_this.uptime();
            });
            this.badgeClass = ko.pureComputed(function () {
                if (!_this.licensed()) {
                    return "state-danger";
                }
                if (_this.disabled()) {
                    return "state-warning";
                }
                if (_this.online()) {
                    return "state-success";
                }
                return "state-offline"; // offline
            });
            this.badgeText = ko.pureComputed(function () {
                if (!_this.licensed()) {
                    return "Unlicensed";
                }
                if (_this.disabled()) {
                    return "Disabled";
                }
                if (_this.uptime()) {
                    return "Online";
                }
                return "Offline";
            });
            this.canNavigateToResource = ko.pureComputed(function () {
                var hasLicense = _this.licensed();
                var enabled = !_this.disabled();
                return hasLicense && enabled;
            });
            this.isCurrentlyActiveResource = ko.pureComputed(function () {
                var currentResource = activeResourceTracker.default.resource();
                if (!currentResource) {
                    return false;
                }
                return currentResource.qualifiedName === _this.qualifiedName;
            });
            this.isLoading = ko.pureComputed(function () {
                return _this.isCurrentlyActiveResource() &&
                    _this.online() === false &&
                    _this.disabled() === false;
            });
        };
        return resourceInfo;
    }());
    return resourceInfo;
});

//# sourceMappingURL=resourceInfo.js.map
