define(["require", "exports", "plugins/router", "models/database/documents/collection", "models/database/documents/recentDocuments", "commands/database/documents/verifyDocumentsIDsCommand", "commands/database/documents/getCollectionsStatsCommand", "common/appUrl", "widgets/virtualGrid/hyperlinkColumn", "common/helpers/database/documentHelpers", "common/starredDocumentsStorage"], function (require, exports, router, collection, recentDocumentsCtr, verifyDocumentsIDsCommand, getCollectionsStatsCommand, appUrl, hyperlinkColumn, documentHelpers, starredDocumentsStorage) {
    "use strict";
    var connectedDocuments = (function () {
        function connectedDocuments(document, db, loadDocument) {
            var _this = this;
            this.searchInput = ko.observable("");
            this.columns = [
                new hyperlinkColumn("id", "href", "", "100%")
            ];
            this.currentDocumentIsStarred = ko.observable(false);
            this.currentTab = ko.observable(connectedDocuments.connectedDocsTabs.related);
            this.recentDocuments = new recentDocumentsCtr();
            this.isRelatedActive = ko.pureComputed(function () { return _this.currentTab() === connectedDocuments.connectedDocsTabs.related; });
            this.isCollectionActive = ko.pureComputed(function () { return _this.currentTab() === connectedDocuments.connectedDocsTabs.collection; });
            this.isRecentActive = ko.pureComputed(function () { return _this.currentTab() === connectedDocuments.connectedDocsTabs.recent; });
            this.isStarredActive = ko.pureComputed(function () { return _this.currentTab() === connectedDocuments.connectedDocsTabs.starred; });
            _.bindAll(this, "toggleStar");
            this.document = document;
            this.db = db;
            this.document.subscribe(function (doc) { return _this.onDocumentLoaded(doc); });
            this.loadDocumentAction = loadDocument;
        }
        connectedDocuments.prototype.fetchCurrentTabDocs = function (skip, take) {
            switch (this.currentTab()) {
                case connectedDocuments.connectedDocsTabs.related:
                    return this.fetchRelatedDocs(skip, take);
                case connectedDocuments.connectedDocsTabs.collection:
                    return this.fetchCollectionDocs(skip, take);
                case connectedDocuments.connectedDocsTabs.recent:
                    return this.fetchRecentDocs(skip, take);
                case connectedDocuments.connectedDocsTabs.starred:
                    return this.fetchStarredDocs(skip, take);
                default: return this.emptyDocResult();
            }
        };
        connectedDocuments.prototype.fetchRelatedDocs = function (skip, take) {
            var _this = this;
            var deferred = $.Deferred();
            var relatedDocumentsCandidates = documentHelpers.findRelatedDocumentsCandidates(this.document());
            var docIDsVerifyCommand = new verifyDocumentsIDsCommand(relatedDocumentsCandidates, this.db(), true, true);
            docIDsVerifyCommand.execute()
                .done(function (verifiedIDs) {
                var connectedDocs = verifiedIDs.map(function (id) { return _this.docIdToConnectedDoc(id); });
                deferred.resolve({
                    items: connectedDocs,
                    totalResultCount: connectedDocs.length
                });
            });
            return deferred.promise();
        };
        connectedDocuments.prototype.fetchCollectionDocs = function (skip, take) {
            var _this = this;
            // Make sure we've got a collection to work with.
            var doc = this.document();
            if (!doc) {
                return this.emptyDocResult();
            }
            // Fetch collection size.
            // Why? Because calling collection.fetchDocuments returns a pagedResultSet with .totalResultCount = 0. :-(
            var collectionName = doc.getEntityName();
            var collectionSizeTask = new getCollectionsStatsCommand(this.db())
                .execute()
                .then(function (stats) { return stats.getCollectionCount(collectionName); });
            // Fetch the chunk of documents.        
            var newCollection = new collection(collectionName, this.db());
            var fetchDocsTask = newCollection.fetchDocuments(skip, take)
                .then(function (result) {
                // Convert the items from document to connectedDocument.
                result.items = result.items.map(function (doc) { return _this.documentToConnectedDoc(doc); });
                return result;
            });
            return $.when(collectionSizeTask, fetchDocsTask)
                .then(function (collectionSize, docsResult) {
                docsResult.totalResultCount = collectionSize;
                return docsResult;
            });
        };
        connectedDocuments.prototype.fetchRecentDocs = function (skip, take) {
            var doc = this.document();
            if (!doc) {
                return this.emptyDocResult();
            }
            var recentDocs = this.recentDocuments.getTopRecentDocuments(this.db(), doc.getId());
            return $.Deferred().resolve({
                items: recentDocs,
                totalResultCount: recentDocs.length
            }).promise();
        };
        connectedDocuments.prototype.fetchStarredDocs = function (skip, take) {
            var _this = this;
            var starredDocIds = starredDocumentsStorage.getStarredDocuments(this.db());
            var starredDocs = starredDocIds.map(function (id) { return _this.docIdToConnectedDoc(id); });
            return $.Deferred().resolve({
                items: starredDocs,
                totalResultCount: starredDocs.length
            }).promise();
        };
        connectedDocuments.prototype.emptyDocResult = function () {
            return $.Deferred().resolve({
                items: [],
                totalResultCount: 0
            }).promise();
        };
        connectedDocuments.prototype.activateRelated = function () {
            this.currentTab(connectedDocuments.connectedDocsTabs.related);
        };
        connectedDocuments.prototype.activateCollection = function () {
            this.currentTab(connectedDocuments.connectedDocsTabs.collection);
        };
        connectedDocuments.prototype.activateRecent = function () {
            this.currentTab(connectedDocuments.connectedDocsTabs.recent);
        };
        connectedDocuments.prototype.activateStarred = function () {
            this.currentTab(connectedDocuments.connectedDocsTabs.starred);
        };
        connectedDocuments.prototype.onDocumentDeleted = function () {
            this.recentDocuments.documentRemoved(this.db(), this.document().getId());
            var previous = this.recentDocuments.getPreviousDocument(this.db());
            if (previous) {
                this.loadDocumentAction(previous);
                router.navigate(appUrl.forEditDoc(previous, this.db()), false);
            }
            else {
                router.navigate(appUrl.forDocuments(null, this.db()));
            }
        };
        connectedDocuments.prototype.toggleStar = function () {
            this.currentDocumentIsStarred(!this.currentDocumentIsStarred());
            starredDocumentsStorage.markDocument(this.db(), this.document().getId(), this.currentDocumentIsStarred());
        };
        connectedDocuments.prototype.onDocumentLoaded = function (document) {
            if (document) {
                this.recentDocuments.appendRecentDocument(this.db(), this.document().getId());
                this.currentDocumentIsStarred(starredDocumentsStorage.isStarred(this.db(), this.document().getId()));
            }
        };
        connectedDocuments.prototype.documentToConnectedDoc = function (doc) {
            return {
                id: doc.getId(),
                href: appUrl.forEditDoc(doc.getId(), this.db())
            };
        };
        connectedDocuments.prototype.docIdToConnectedDoc = function (docId) {
            return {
                id: docId,
                href: appUrl.forEditDoc(docId, this.db())
            };
        };
        return connectedDocuments;
    }());
    connectedDocuments.connectedDocsTabs = {
        related: "related",
        collection: "collection",
        recent: "recent",
        starred: "starred"
    };
    return connectedDocuments;
});

//# sourceMappingURL=editDocumentConnectedDocuments.js.map
