define(["require", "exports", "common/helpers/graph/graphHelper", "d3", "rbush"], function (require, exports, graphHelper, d3, rbush) {
    "use strict";
    var pageItem = (function () {
        function pageItem(pageNumber, parent) {
            this.pageNumber = pageNumber;
            this.parent = parent;
            this.incomingLinesCount = 0;
        }
        pageItem.prototype.getSourceConnectionPoint = function () {
            return [this.x + pageItem.pageWidth / 2, this.y];
        };
        pageItem.prototype.getTargetConnectionPoint = function () {
            return [this.x + pageItem.pageWidth / 2, this.y + pageItem.pageHeight];
        };
        pageItem.prototype.getGlobalTargetConnectionPoint = function () {
            var localConnectionPoint = this.getTargetConnectionPoint();
            return [localConnectionPoint[0] + this.parent.x, localConnectionPoint[1] + this.parent.y];
        };
        // Return the X offset from where to start drawing the first incoming line to the pageItem
        pageItem.prototype.getGlobalTargetConnectionPointXOffset = function () {
            if (this.incomingLinesCount === 1)
                return 0;
            var segmentSize = 4;
            var xOffset = Math.floor(this.incomingLinesCount / 2) * segmentSize;
            if (this.incomingLinesCount % 2 === 0) {
                xOffset -= 2;
            }
            return xOffset;
        };
        return pageItem;
    }());
    pageItem.pageWidth = 20;
    pageItem.pageHeight = 20;
    var reduceTreeItem = (function () {
        function reduceTreeItem(tree) {
            this.itemsAsDepth = new Map(); // items after filtering
            this.width = 0;
            this.height = 0;
            this.extraLeftPadding = 0; //used when text (reduce key) is longer than contents
            this.tree = tree;
            this.name = tree.Name;
            this.displayName = tree.DisplayName;
            this.depth = tree.Depth;
            this.countItemsPerDepth();
        }
        reduceTreeItem.prototype.countItemsPerDepth = function () {
            var _this = this;
            this.itemsCountAtDepth = new Array(this.depth);
            var countEntries = function (depth, node) {
                _this.itemsCountAtDepth[depth] = (_this.itemsCountAtDepth[depth] || 0) + 1;
                if (node.Children && node.Children.length) {
                    for (var i = 0; i < node.Children.length; i++) {
                        countEntries(depth + 1, node.Children[i]);
                    }
                }
            };
            countEntries(0, this.tree.Root);
        };
        reduceTreeItem.prototype.filterAndLayoutVisibleItems = function (documents) {
            this.cleanCache(documents);
            this.filterVisibleItems(documents);
            this.layout();
        };
        reduceTreeItem.prototype.cleanCache = function (documents) {
            this.itemsAsDepth.clear();
        };
        reduceTreeItem.prototype.filterVisibleItems = function (documents) {
            var _this = this;
            //TODO: for now display all
            var filterAtDepth = function (depth, node, aggregation) {
                if (!_this.itemsAsDepth.has(depth)) {
                    _this.itemsAsDepth.set(depth, []);
                }
                var items = _this.itemsAsDepth.get(depth);
                var item = new pageItem(node.PageNumber, _this);
                item.aggregation = aggregation;
                items.push(item);
                if (node.Children) {
                    for (var i = 0; i < node.Children.length; i++) {
                        filterAtDepth(depth + 1, node.Children[i], item);
                    }
                }
                if (node.Entries) {
                    var uniqueSources = new Set(node.Entries.filter(function (x) { return x.Source; }).map(function (x) { return x.Source; }));
                    uniqueSources.forEach(function (source) {
                        var matchingDoc = documents.find(function (d) { return d.name === source; });
                        item.incomingLinesCount++;
                        matchingDoc.connectedPages.push(item);
                    });
                }
            };
            filterAtDepth(0, this.tree.Root, null);
        };
        reduceTreeItem.prototype.layout = function () {
            this.calculateTreeDimensions();
            var maxItems = d3.max(this.itemsCountAtDepth);
            var pagesTotalWidth = this.getPagesOnlyWidth(maxItems);
            var yStart = reduceTreeItem.margins.treeMargin +
                reduceTreeItem.margins.titleHeight +
                reduceTreeItem.margins.treeMargin;
            var yOffset = pageItem.pageHeight +
                reduceTreeItem.margins.betweenPagesVerticalPadding;
            for (var depth = 0; depth < this.depth; depth++) {
                var items = this.itemsAsDepth.get(depth);
                var startAndOffset = graphHelper
                    .computeStartAndOffset(pagesTotalWidth, items.length, pageItem.pageWidth);
                var xOffset = startAndOffset.offset;
                var xStart = reduceTreeItem.margins.treeMargin +
                    visualizerGraphGlobal.totalEntriesWidth(maxItems) +
                    reduceTreeItem.margins.treeMargin +
                    this.extraLeftPadding +
                    startAndOffset.start;
                for (var i = 0; i < items.length; i++) {
                    var item = items[i];
                    item.y = yStart;
                    item.x = xStart;
                    xStart += xOffset;
                }
                yStart += yOffset;
            }
        };
        reduceTreeItem.prototype.calculateTreeDimensions = function () {
            var height = reduceTreeItem.margins.treeMargin +
                reduceTreeItem.margins.titleHeight +
                reduceTreeItem.margins.treeMargin;
            height += (this.depth * pageItem.pageHeight) +
                (this.depth - 1) * reduceTreeItem.margins.betweenPagesVerticalPadding;
            height += reduceTreeItem.margins.treeMargin;
            this.height = height;
            var maxItems = d3.max(this.itemsCountAtDepth);
            var width = reduceTreeItem.margins.treeMargin +
                visualizerGraphGlobal.totalEntriesWidth(maxItems) +
                reduceTreeItem.margins.treeMargin;
            width += this.getPagesOnlyWidth(maxItems);
            width += reduceTreeItem.margins.treeMargin;
            var estimatedTextWidth = this.estimateReduceKeyWidth() + 2 * reduceTreeItem.margins.treeMargin;
            if (estimatedTextWidth > width) {
                this.extraLeftPadding = (estimatedTextWidth - width) / 2;
                width = estimatedTextWidth;
            }
            else {
                this.extraLeftPadding = 0;
            }
            this.width = width;
        };
        reduceTreeItem.prototype.estimateReduceKeyWidth = function () {
            return this.displayName.length * 6;
        };
        reduceTreeItem.prototype.getPagesOnlyWidth = function (maxItems) {
            return maxItems * pageItem.pageWidth +
                (maxItems + 1) * reduceTreeItem.margins.betweenPagesHorizontalPadding;
        };
        return reduceTreeItem;
    }());
    /*
    Top-bottom: treeMargin, titleHeight, treeMargin, pageHeight, betweenPagesVerticalPadding, ..., pageHeight, treeMargin
    Left-right: treeMargin, <dynamic text margin>, treeMargin, pageWidth, betweenPagesHorizontalPadding, ..., pageWidth, treeMargin
    */
    reduceTreeItem.margins = {
        treeMargin: 20,
        betweenPagesHorizontalPadding: 13,
        betweenPagesVerticalPadding: 50,
        titleHeight: 20
    };
    var documentItem = (function () {
        function documentItem(name) {
            this.connectedPages = [];
            this.name = name;
        }
        documentItem.prototype.getSourceConnectionPoint = function () {
            return [this.x + this.width / 2, this.y];
        };
        return documentItem;
    }());
    documentItem.margins = {
        minMarginBetweenDocumentNames: 30,
        badgePadding: 30
    };
    var hitTest = (function () {
        function hitTest() {
            this.rTree = rbush();
        }
        hitTest.prototype.init = function (onPageClicked) {
            this.onPageClicked = onPageClicked;
        };
        hitTest.prototype.registerPage = function (page) {
            this.rTree.insert({
                minX: page.x,
                maxX: page.x + page.width,
                minY: page.y,
                maxY: page.y + page.height,
                actionType: "pageClicked",
                arg: page
            });
        };
        hitTest.prototype.reset = function () {
            this.rTree.clear();
        };
        hitTest.prototype.onClick = function (x, y) {
            var items = this.findItems(x, y);
            for (var i = 0; i < items.length; i++) {
                var item = items[i];
                this.onPageClicked(item.arg);
            }
        };
        hitTest.prototype.findItems = function (x, y) {
            return this.rTree.search({
                minX: x,
                maxX: x,
                minY: y,
                maxY: y
            });
        };
        return hitTest;
    }());
    var visualizerGraphGlobal = (function () {
        function visualizerGraphGlobal() {
            this.nextColorIndex = 0;
            this.hitTest = new hitTest();
            this.savedZoomStatus = {
                scale: 1,
                translate: [0, 0]
            };
            this.totalWidth = 1500; //TODO: use dynamic value
            this.totalHeight = 700; //TODO: use dynamic value
            this.documents = [];
            this.reduceTrees = [];
            this.dataWidth = 0; // total width of all virtual elements
            this.dataHeight = 0; // total heigth of all virtual elements
        }
        visualizerGraphGlobal.prototype.addDocument = function (documentName) {
            var document = new documentItem(documentName);
            this.documents.push(document);
        };
        visualizerGraphGlobal.prototype.addTrees = function (result) {
            var _loop_1 = function (i) {
                var reduceTree = result[i];
                var existingTree = this_1.reduceTrees.find(function (x) { return x.name === reduceTree.Name; });
                if (!existingTree) {
                    var newTree = new reduceTreeItem(reduceTree);
                    this_1.reduceTrees.push(newTree);
                }
            };
            var this_1 = this;
            for (var i = 0; i < result.length; i++) {
                _loop_1(i);
            }
            this.layout();
        };
        visualizerGraphGlobal.prototype.zoomToDocument = function (documentName) {
            var documentNameLowerCase = documentName.toLowerCase();
            var doc = this.documents.find(function (x) { return x.name.toLowerCase() === documentNameLowerCase; });
            if (doc) {
                var requestedTranslation = [-doc.x + this.totalWidth / 2 - doc.width / 2, 0];
                if (this.documents.length === 1) {
                    // we don't want animation on first element
                    this.zoom.translate(requestedTranslation).scale(1).event(this.canvas);
                }
                else {
                    this.canvas
                        .transition()
                        .duration(500)
                        .call(this.zoom.translate(requestedTranslation).scale(1).event);
                }
            }
        };
        visualizerGraphGlobal.prototype.reset = function () {
            this.documents = [];
            this.reduceTrees = [];
            this.draw();
        };
        visualizerGraphGlobal.prototype.init = function (goToDetailsCallback) {
            var _this = this;
            this.goToDetailsCallback = goToDetailsCallback;
            var container = d3.select("#visualizerContainer");
            this.canvas = container
                .append("canvas")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight);
            this.svg = container
                .append("svg")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight);
            this.xScale = d3.scale.linear()
                .domain([0, this.totalWidth])
                .range([0, this.totalWidth]);
            this.yScale = d3.scale.linear()
                .domain([0, this.totalHeight])
                .range([0, this.totalHeight]);
            this.zoom = d3.behavior.zoom()
                .x(this.xScale)
                .y(this.yScale)
                .on("zoom", function () { return _this.onZoom(); });
            this.svg
                .append("svg:rect")
                .attr("class", "pane")
                .attr("width", this.totalWidth)
                .attr("height", this.totalHeight)
                .call(this.zoom)
                .call(function (d) { return _this.setupEvents(d); });
            this.hitTest.init(function (item) { return _this.onPageClicked(item); });
        };
        visualizerGraphGlobal.prototype.getDocumentsColors = function () {
            var documentsColorsSetup = [];
            for (var i = 0; i < this.documents.length; i++) {
                var doc = this.documents[i];
                documentsColorsSetup.push({ docName: doc.name, docColor: doc.color });
            }
            return documentsColorsSetup;
        };
        visualizerGraphGlobal.prototype.setupEvents = function (selection) {
            var _this = this;
            selection.on("dblclick.zoom", null);
            selection.on("click", function () { return _this.onClick(); });
        };
        visualizerGraphGlobal.prototype.onZoom = function () {
            this.draw();
        };
        visualizerGraphGlobal.prototype.onPageClicked = function (item) {
            var _this = this;
            this.saveCurrentZoom();
            var requestedScale = Math.min(this.totalWidth / item.width, this.totalHeight / item.height);
            var extraXOffset = (this.totalWidth - requestedScale * item.width) / 2;
            var requestedTranslation = [-item.x * requestedScale + extraXOffset, -item.y * requestedScale];
            this.canvas
                .transition()
                .duration(500)
                .call(this.zoom.translate(requestedTranslation).scale(requestedScale).event)
                .style('opacity', 0)
                .each("end", function () {
                _this.toggleUiElements(false);
                _this.goToDetailsCallback(item.name);
            });
        };
        visualizerGraphGlobal.prototype.restoreView = function () {
            this.toggleUiElements(true);
            this.canvas
                .transition()
                .duration(500)
                .call(this.zoom.translate(this.savedZoomStatus.translate).scale(this.savedZoomStatus.scale).event)
                .style('opacity', 1);
        };
        visualizerGraphGlobal.prototype.saveCurrentZoom = function () {
            this.savedZoomStatus.scale = this.zoom.scale();
            this.savedZoomStatus.translate = this.zoom.translate();
        };
        visualizerGraphGlobal.prototype.onClick = function () {
            var clickLocation = d3.mouse(this.canvas.node());
            if (d3.event.defaultPrevented) {
                return;
            }
            this.hitTest.onClick(this.xScale.invert(clickLocation[0]), this.yScale.invert(clickLocation[1]));
        };
        visualizerGraphGlobal.prototype.cleanLayoutCache = function () {
            for (var i = 0; i < this.documents.length; i++) {
                this.documents[i].connectedPages = [];
            }
        };
        visualizerGraphGlobal.prototype.layout = function () {
            this.cleanLayoutCache();
            // layout children first
            for (var i = 0; i < this.reduceTrees.length; i++) {
                var tree = this.reduceTrees[i];
                tree.filterAndLayoutVisibleItems(this.documents);
            }
            var maxTreeHeight = d3.max(this.reduceTrees, function (x) { return x.height; });
            var currentX = visualizerGraphGlobal.margins.globalMargin;
            for (var i = 0; i < this.reduceTrees.length; i++) {
                var tree = this.reduceTrees[i];
                tree.x = currentX;
                tree.y = visualizerGraphGlobal.margins.globalMargin + (maxTreeHeight - tree.height);
                currentX += tree.width;
                currentX += visualizerGraphGlobal.margins.betweenPagesWidth;
            }
            currentX -= visualizerGraphGlobal.margins.betweenPagesWidth;
            currentX += visualizerGraphGlobal.margins.globalMargin;
            this.dataWidth = currentX;
            var documentNamesYStart = visualizerGraphGlobal.margins.globalMargin +
                maxTreeHeight +
                visualizerGraphGlobal.margins.betweenPagesAndDocuments;
            var height = documentNamesYStart
                + 100 //TODO: it is space for document names
                + visualizerGraphGlobal.margins.globalMargin; // top and bottom margin
            this.dataHeight = height;
            this.layoutDocuments(documentNamesYStart);
            this.registerHitAreas();
        };
        visualizerGraphGlobal.prototype.layoutDocuments = function (yStart) {
            var totalWidth = 0;
            for (var i = 0; i < this.documents.length; i++) {
                var doc = this.documents[i];
                var documentNameWidthEstimation = function (text) { return text.length * 9; };
                doc.width = documentItem.margins.badgePadding * 2 + documentNameWidthEstimation(doc.name);
                doc.height = 35;
                doc.y = yStart;
                totalWidth += doc.width;
            }
            totalWidth += this.documents.length * (documentItem.margins.minMarginBetweenDocumentNames + 1);
            var extraItemPadding = 0;
            if (totalWidth > this.dataWidth) {
            }
            var avgXValues = [];
            for (var currentDoc = 0; currentDoc < this.documents.length; currentDoc++) {
                var doc = this.documents[currentDoc];
                // Calc document x value as average of connectedPages
                var totalXValuesOfConnectedPages = doc.connectedPages.reduce(function (a, b) { return a + b.parent.x; }, 0);
                doc.x = totalXValuesOfConnectedPages / doc.connectedPages.length;
                avgXValues.push({ docIndex: currentDoc, avgXVal: doc.x });
            }
            // Sort the avg x values 
            avgXValues.sort(function (a1, a2) { return a1.avgXVal - a2.avgXVal; });
            // set color of 1'st document
            this.nextColorIndex = 0;
            this.documents[0].color = this.getNextColor();
            // Order documents according to the sorted values and set documents colors 
            var xPadding = 20;
            for (var i = 1; i < avgXValues.length; i++) {
                var doc = this.documents[avgXValues[i].docIndex];
                // Check for collisions..
                var previousX = avgXValues[i - 1].avgXVal;
                if ((doc.x >= previousX) && (doc.x <= previousX + doc.width) ||
                    (doc.x + doc.width >= previousX) && (doc.x + doc.width <= previousX + doc.width)) {
                    doc.x = previousX + doc.width + xPadding;
                    avgXValues[i].avgXVal = doc.x;
                }
                doc.color = this.getNextColor();
            }
        };
        visualizerGraphGlobal.prototype.registerHitAreas = function () {
            this.hitTest.reset();
            for (var i = 0; i < this.reduceTrees.length; i++) {
                this.hitTest.registerPage(this.reduceTrees[i]);
            }
        };
        visualizerGraphGlobal.prototype.draw = function () {
            var canvas = this.canvas.node();
            var ctx = canvas.getContext("2d");
            ctx.clearRect(0, 0, this.totalWidth, this.totalHeight);
            ctx.save();
            var translation = this.zoom.translate();
            ctx.translate(translation[0], translation[1]);
            ctx.scale(this.zoom.scale(), this.zoom.scale());
            for (var i = 0; i < this.reduceTrees.length; i++) {
                this.drawTree(ctx, this.reduceTrees[i]);
            }
            for (var i = 0; i < this.documents.length; i++) {
                var doc = this.documents[i];
                this.drawDocument(ctx, doc);
            }
            this.drawDocumentConnections(ctx);
            ctx.restore();
        };
        visualizerGraphGlobal.prototype.drawTree = function (ctx, tree) {
            ctx.fillStyle = "#2c3333";
            ctx.fillRect(tree.x, tree.y, tree.width, tree.height);
            ctx.save();
            try {
                ctx.translate(tree.x, tree.y);
                // text
                ctx.font = "10px Lato";
                ctx.textAlign = "center";
                ctx.fillStyle = "#686f6f";
                ctx.fillText(tree.displayName, tree.width / 2, reduceTreeItem.margins.treeMargin, tree.width);
                // total entries
                var totalEntiresY = reduceTreeItem.margins.treeMargin +
                    reduceTreeItem.margins.titleHeight +
                    reduceTreeItem.margins.treeMargin;
                var totalEntriesOffset = pageItem.pageHeight +
                    reduceTreeItem.margins.betweenPagesVerticalPadding;
                for (var i = 1; i < tree.depth; i++) {
                    totalEntiresY += totalEntriesOffset;
                    var items = tree.itemsCountAtDepth[i];
                    ctx.font = "18px Lato";
                    ctx.textAlign = "right";
                    ctx.fillText(items.toString(), reduceTreeItem.margins.treeMargin + visualizerGraphGlobal.totalEntriesWidth(items), totalEntiresY + 18 /* font size */);
                }
                this.drawPages(ctx, tree);
            }
            finally {
                ctx.restore();
            }
        };
        visualizerGraphGlobal.prototype.drawPages = function (ctx, tree) {
            ctx.fillStyle = "#008cc9";
            ctx.strokeStyle = "#686f6f";
            tree.itemsAsDepth.forEach(function (globalItems) {
                for (var i = 0; i < globalItems.length; i++) {
                    var item = globalItems[i];
                    ctx.fillRect(item.x, item.y, pageItem.pageWidth, pageItem.pageHeight);
                    if (item.aggregation) {
                        var sourcePoint = item.getSourceConnectionPoint();
                        var targetPoint = item.aggregation.getTargetConnectionPoint();
                        graphHelper.drawBezierDiagonal(ctx, sourcePoint, targetPoint, true);
                    }
                }
            });
        };
        visualizerGraphGlobal.prototype.drawDocumentConnections = function (ctx) {
            ctx.lineWidth = 2;
            var straightLine = 8;
            var targetXValuesDrawnOnScreen = [];
            for (var i = 0; i < this.documents.length; i++) {
                var doc = this.documents[i];
                ctx.strokeStyle = doc.color;
                straightLine += 5;
                for (var j = 0; j < doc.connectedPages.length; j++) {
                    var page = doc.connectedPages[j];
                    var targetXY = page.getGlobalTargetConnectionPoint();
                    var targetXOffset = targetXY[0] - page.getGlobalTargetConnectionPointXOffset();
                    while (_.includes(targetXValuesDrawnOnScreen, targetXOffset)) {
                        targetXOffset += 4;
                    }
                    targetXY[0] = targetXOffset;
                    var sourceXY = doc.getSourceConnectionPoint();
                    if (straightLine > 60) {
                        straightLine = 11;
                    }
                    this.drawStraightLines(ctx, sourceXY, targetXY, straightLine);
                    // Save x value of target to avoid collisions
                    targetXValuesDrawnOnScreen.push(targetXY[0]);
                }
            }
        };
        visualizerGraphGlobal.prototype.drawStraightLines = function (ctx, source, target, straightLine) {
            ctx.beginPath();
            var m = (source[1] + target[1]) / 2;
            if (source[1] < target[1]) {
                ctx.moveTo(source[0], source[1]);
                ctx.lineTo(source[0], source[1] + straightLine);
                ctx.lineTo(target[0], source[1] + straightLine);
                ctx.lineTo(target[0], target[1]);
                ctx.stroke();
            }
            else {
                ctx.moveTo(source[0], source[1]);
                ctx.lineTo(source[0], source[1] - straightLine);
                ctx.lineTo(target[0], source[1] - straightLine);
                ctx.lineTo(target[0], target[1]);
                ctx.stroke();
            }
            // Draw arrow
            var halfWidth = 6;
            var height = 8;
            ctx.beginPath();
            ctx.moveTo(target[0] - halfWidth, target[1] + height);
            ctx.lineTo(target[0], target[1]);
            ctx.lineTo(target[0] + halfWidth, target[1] + height);
            ctx.stroke();
        };
        /* TODO striped curves
    
        ctx.lineWidth = 2;
    ctx.strokeStyle = "black";
    ctx.setLineDash([16, 16]);
    ctx.lineDashOffset = 0;
    
    ctx.beginPath();
    ctx.moveTo(20,20);
    
    ctx.bezierCurveTo(20,100,200,100,200,20);
    ctx.stroke();
    
    
    ctx.strokeStyle = "red"
    ctx.setLineDash([16, 16]);
    ctx.lineDashOffset = 16;
    
    ctx.beginPath();
    ctx.moveTo(20,20);
    ctx.bezierCurveTo(20,100,200,100,200,20);
    ctx.stroke();*/
        visualizerGraphGlobal.prototype.drawDocument = function (ctx, docItem) {
            ctx.fillStyle = docItem.color;
            ctx.fillRect(docItem.x, docItem.y, docItem.width, docItem.height);
            ctx.textAlign = "center";
            ctx.textBaseline = "middle";
            ctx.font = "18px Lato";
            ctx.fillStyle = "black";
            ctx.fillText(docItem.name, docItem.x + docItem.width / 2, docItem.y + docItem.height / 2);
        };
        visualizerGraphGlobal.prototype.toggleUiElements = function (show) {
            this.svg.style("display", show ? "block" : "none");
            this.canvas.style("display", show ? "block" : "none");
        };
        visualizerGraphGlobal.totalEntriesWidth = function (entries) {
            return Math.ceil(Math.log10(entries)) * 10;
        };
        visualizerGraphGlobal.prototype.getNextColor = function () {
            var color = visualizerGraphGlobal.documentColors[this.nextColorIndex % visualizerGraphGlobal.documentColors.length];
            this.nextColorIndex++;
            return color;
        };
        return visualizerGraphGlobal;
    }());
    visualizerGraphGlobal.documentColors = ["#2196f5", "#ef6c5a", "#80ced0", "#9ccd64", "#f06292", "#7f45e6", "#fea724", "#01acc0"];
    visualizerGraphGlobal.margins = {
        betweenPagesWidth: 30,
        betweenPagesAndDocuments: 80,
        globalMargin: 30
    };
    return visualizerGraphGlobal;
});

//# sourceMappingURL=visualizerGraphGlobal.js.map
