/// <reference path="../../typings/tsd.d.ts"/>
define(["require", "exports", "common/appUrl", "common/shell/activeResourceTracker", "plugins/router", "durandal/app", "common/oauthContext", "common/changesContext", "viewmodels/common/confirmationDialog", "commands/database/documents/saveDocumentCommand", "models/database/documents/document", "common/downloader", "common/shell/resourcesManager", "common/helpers/text/pluralizeHelpers", "common/eventsCollector"], function (require, exports, appUrl, activeResourceTracker, router, app, oauthContext, changesContext, confirmationDialog, saveDocumentCommand, document, downloader, resourcesManager, pluralizeHelpers, eventsCollector) {
    "use strict";
    /*
     * Base view model class that provides basic view model services, such as tracking the active resource and providing a means to add keyboard shortcuts.
    */
    var viewModelBase = (function () {
        function viewModelBase() {
            var _this = this;
            this.activeDatabase = activeResourceTracker.default.database;
            this.activeFilesystem = activeResourceTracker.default.fileSystem;
            this.activeCounterStorage = activeResourceTracker.default.counterStorage;
            this.activeTimeSeries = activeResourceTracker.default.timeSeries;
            this.activeResource = activeResourceTracker.default.resource;
            this.downloader = new downloader();
            this.isBusy = ko.observable(false);
            this.resourcesManager = resourcesManager.default;
            this.keyboardShortcutDomContainers = [];
            this.notifications = [];
            this.postboxSubscriptions = [];
            this.isAttached = false;
            this.pluralize = pluralizeHelpers.pluralize;
            this.changesContext = changesContext.default;
            this.dirtyFlag = new ko.DirtyFlag([]);
            this.currentHelpLink = ko.observable().subscribeTo('globalHelpLink', true);
            this.beforeUnloadListener = function (e) {
                var isDirty = _this.dirtyFlag().isDirty();
                if (isDirty) {
                    var message = "You have unsaved data.";
                    e = e || window.event;
                    // For IE and Firefox
                    if (e) {
                        e.returnValue = message;
                    }
                    // For Safari
                    return message;
                }
            };
            this.appUrls = appUrl.forCurrentDatabase();
            eventsCollector.default.reportViewModel(this);
        }
        viewModelBase.prototype.bindToCurrentInstance = function () {
            var methods = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                methods[_i] = arguments[_i];
            }
            _.bindAll.apply(_, [this].concat(methods));
        };
        viewModelBase.prototype.canActivate = function (args) {
            var self = this;
            setTimeout(function () { return viewModelBase.showSplash(self.isAttached === false); }, 700);
            this.downloader.reset();
            this.resourcesManager.activateBasedOnCurrentUrl();
            return true;
        };
        viewModelBase.prototype.activate = function (args, isShell) {
            var _this = this;
            if (isShell === void 0) { isShell = false; }
            // create this ko.computed once to avoid creation and subscribing every 50 ms - thus creating memory leak.
            var adminArea = this.appUrls.isAreaActive("admin");
            oauthContext.enterApiKeyTask.done(function () {
                if (isShell || adminArea())
                    return;
                _this.changesContext
                    .afterConnection
                    .done(function () {
                    (_a = _this.notifications).push.apply(_a, _this.createNotifications());
                    var _a;
                });
            });
            this.postboxSubscriptions = this.createPostboxSubscriptions();
            this.modelPollingStart();
            ko.postbox.publish("SetRawJSONUrl", "");
            this.updateHelpLink(null); // clean link
        };
        viewModelBase.prototype.getPageHostDimenensions = function () {
            var $pageHostRoot = $("#page-host-root");
            return [$pageHostRoot.width(), $pageHostRoot.height()];
        };
        viewModelBase.prototype.attached = function () {
            window.addEventListener("beforeunload", this.beforeUnloadListener, false);
            this.isAttached = true;
            viewModelBase.showSplash(false);
        };
        viewModelBase.prototype.rightPanelSetup = function () {
            var $pageHostRoot = $("#page-host-root");
            var hasRightPanel = !!$("#right-options-panel", $pageHostRoot).length;
            $pageHostRoot.toggleClass("enable-right-options-panel", hasRightPanel);
        };
        viewModelBase.prototype.compositionComplete = function () {
            this.rightPanelSetup();
            this.dirtyFlag().reset(); //Resync Changes
        };
        viewModelBase.prototype.canDeactivate = function (isClose) {
            var _this = this;
            var isDirty = this.dirtyFlag().isDirty();
            if (isDirty) {
                var discard = "Discard changes";
                var stay = "Stay on this page";
                var discardStayResult_1 = $.Deferred();
                var confirmation = this.confirmationMessage("Unsaved changes", "You have unsaved changes. How do you want to proceed?", [discard, stay], true);
                confirmation.done(function (result) {
                    if (!result.can) {
                        _this.dirtyFlag().reset();
                    }
                    result.can = !result.can;
                    discardStayResult_1.resolve(result);
                });
                return discardStayResult_1;
            }
            return true;
        };
        viewModelBase.prototype.detached = function () {
            this.currentHelpLink.unsubscribeFrom("currentHelpLink");
            this.cleanupNotifications();
            this.cleanupPostboxSubscriptions();
            window.removeEventListener("beforeunload", this.beforeUnloadListener, false);
            this.isAttached = true;
            viewModelBase.showSplash(false);
        };
        viewModelBase.prototype.deactivate = function () {
            var _this = this;
            this.keyboardShortcutDomContainers.forEach(function (el) { return _this.removeKeyboardShortcuts(el); });
            this.modelPollingStop();
            this.isAttached = true;
            viewModelBase.showSplash(false);
        };
        viewModelBase.prototype.createNotifications = function () {
            return [];
        };
        viewModelBase.prototype.cleanupNotifications = function () {
            this.notifications.forEach(function (notification) { return notification.off(); });
            this.notifications = [];
        };
        viewModelBase.prototype.addNotification = function (subscription) {
            this.notifications.push(subscription);
        };
        viewModelBase.prototype.removeNotification = function (subscription) {
            _.pull(this.notifications, subscription);
        };
        viewModelBase.prototype.createPostboxSubscriptions = function () {
            return [];
        };
        viewModelBase.prototype.cleanupPostboxSubscriptions = function () {
            this.postboxSubscriptions.forEach(function (subscription) { return subscription.dispose(); });
            this.postboxSubscriptions = [];
        };
        /*
         * Creates a keyboard shortcut local to the specified element and its children.
         * The shortcut will be removed as soon as the view model is deactivated.
         * Also defines shortcut for ace editor, if ace editor was received
         */
        viewModelBase.prototype.createKeyboardShortcut = function (keys, handler, elementSelector) {
            jwerty.key(keys, function (e) {
                e.preventDefault();
                handler();
            }, this, elementSelector);
            if (!_.includes(this.keyboardShortcutDomContainers, elementSelector)) {
                this.keyboardShortcutDomContainers.push(elementSelector);
            }
        };
        viewModelBase.prototype.removeKeyboardShortcuts = function (elementSelector) {
            $(elementSelector).unbind('keydown.jwerty');
        };
        /*
         * Navigates to the specified URL, recording a navigation event in the browser's history.
         */
        viewModelBase.prototype.navigate = function (url) {
            router.navigate(url);
        };
        /*
         * Navigates by replacing the current URL. It does not record a new entry in the browser's navigation history.
         */
        viewModelBase.prototype.updateUrl = function (url) {
            var options = {
                replace: true,
                trigger: false
            };
            router.navigate(url, options);
        };
        viewModelBase.prototype.modelPolling = function () {
            return null;
        };
        viewModelBase.prototype.pollingWithContinuation = function () {
            var _this = this;
            var poolPromise = this.modelPolling();
            if (poolPromise) {
                poolPromise.always(function () {
                    viewModelBase.modelPollingHandle = setTimeout(function () {
                        viewModelBase.modelPollingHandle = null;
                        _this.pollingWithContinuation();
                    }, 5000);
                });
            }
        };
        viewModelBase.prototype.modelPollingStart = function () {
            // clear previous pooling handle (if any)
            if (viewModelBase.modelPollingHandle) {
                this.modelPollingStop();
            }
            this.pollingWithContinuation();
        };
        viewModelBase.prototype.modelPollingStop = function () {
            clearTimeout(viewModelBase.modelPollingHandle);
            viewModelBase.modelPollingHandle = null;
        };
        viewModelBase.prototype.confirmationMessage = function (title, confirmationMessage, options, forceRejectWithResolve) {
            if (options === void 0) { options = ["No", "Yes"]; }
            if (forceRejectWithResolve === void 0) { forceRejectWithResolve = false; }
            var viewTask = $.Deferred();
            app.showBootstrapDialog(new confirmationDialog(confirmationMessage, title, options))
                .done(function (answer) {
                var isConfirmed = answer === _.last(options);
                if (isConfirmed) {
                    viewTask.resolve({ can: true });
                }
                else if (!forceRejectWithResolve) {
                    viewTask.reject();
                }
                else {
                    viewTask.resolve({ can: false });
                }
            });
            return viewTask;
        };
        viewModelBase.prototype.canContinueIfNotDirty = function (title, confirmationMessage) {
            var deferred = $.Deferred();
            var isDirty = this.dirtyFlag().isDirty();
            if (isDirty) {
                this.confirmationMessage(title, confirmationMessage)
                    .done(function () { return deferred.resolve(); });
            }
            else {
                deferred.resolve();
            }
            return deferred;
        };
        viewModelBase.prototype.continueTest = function () {
            var doc = document.empty();
            new saveDocumentCommand("Debug/Done", doc, this.activeDatabase(), false)
                .execute()
                .done(function () { return viewModelBase.hasContinueTestOption(false); });
        };
        viewModelBase.prototype.updateHelpLink = function (hash) {
            if (hash === void 0) { hash = null; }
            if (hash) {
                var version = viewModelBase.clientVersion();
                if (version) {
                    var href = "http://ravendb.net/l/" + hash + "/" + version + "/";
                    ko.postbox.publish('globalHelpLink', href);
                    return;
                }
                var subscribtion = viewModelBase.clientVersion.subscribe(function (v) {
                    var href = "http://ravendb.net/l/" + hash + "/" + v + "/";
                    ko.postbox.publish('globalHelpLink', href);
                    if (subscribtion) {
                        subscribtion.dispose();
                    }
                });
            }
            else {
                ko.postbox.publish('globalHelpLink', null);
            }
        };
        viewModelBase.prototype.isValid = function (context, showErrors) {
            if (showErrors === void 0) { showErrors = true; }
            if (context.isValid()) {
                return true;
            }
            else {
                if (showErrors) {
                    context.errors.showAllMessages();
                }
                return false;
            }
        };
        return viewModelBase;
    }());
    viewModelBase.showSplash = ko.observable(false);
    //holds full studio version eg. 4.0.40000
    viewModelBase.clientVersion = ko.observable();
    viewModelBase.hasContinueTestOption = ko.observable(false);
    return viewModelBase;
});

//# sourceMappingURL=viewModelBase.js.map
