/// <reference path="../../../typings/tsd.d.ts"/>
define(["require", "exports", "widgets/virtualGrid/virtualRow", "widgets/virtualGrid/virtualGridConfigDefaults", "widgets/virtualGrid/textColumn", "widgets/virtualGrid/checkedColumn"], function (require, exports, virtualRow, virtualGridConfigDefaults, textColumn, checkedColumn) {
    "use strict";
    var virtualGrid = (function () {
        function virtualGrid(params) {
            var _this = this;
            this.items = []; // The items loaded asynchronously.
            this.totalItemCount = null;
            this.virtualRows = []; // These are the fixed number of elements that get displayed on screen. Each virtual row displays an element from .items array. As the user scrolls, rows will be recycled to represent different items.
            this.virtualHeight = ko.observable(0);
            this.scrollAnimationFrameHandle = 0;
            this.isLoading = ko.observable(false);
            this.queuedFetch = null;
            this.columns = ko.observableArray();
            this.isSelectAllChecked = ko.observable(false);
            this.isGridVisible = false;
            this.selectedIndices = [];
            this.renderHandle = 0;
            this.settings = new virtualGridConfigDefaults();
            this.gridId = "vg-" + (1 + Math.random()).toString().replace(".", "");
            // Configure the grid using the parameters passed in from HTML.
            $.extend(this.settings, params);
            this.initializeColumns();
            this.isSelectAllChecked.subscribe(function (allSelected) { return _this.selectAllChanged(allSelected); });
            if (params.resetItems) {
                params.resetItems.subscribe(function () { return _this.resetItems(); });
            }
            this.fetchItems(0, 100);
        }
        // Called by Knockout once the grid has been rendered.
        virtualGrid.prototype.afterRender = function () {
            this.initializeUIElements();
        };
        virtualGrid.prototype.initializeUIElements = function () {
            var _this = this;
            this.$gridElement = this.findGridElement();
            this.gridElementHeight = this.$gridElement.height();
            this.$gridElement.on("click", function (e) { return _this.gridClicked(e); });
            this.$viewportElement = this.$gridElement.find(virtualGrid.viewportSelector);
            this.initializeVirtualRows();
            this.$gridElement.find(virtualGrid.viewportSelector).on("scroll", function () { return _this.gridScrolled(); });
        };
        virtualGrid.prototype.initializeVirtualRows = function () {
            this.virtualRows = this.createVirtualRows();
            this.$viewportElement
                .find(virtualGrid.viewportScrollerSelector)
                .empty()
                .append(this.virtualRows.map(function (r) { return r.element[0]; }));
        };
        virtualGrid.prototype.createVirtualRows = function () {
            var height = Math.max(100, this.gridElementHeight);
            var rowsNeededToCoverViewport = Math.ceil(height / virtualRow.height);
            var desiredRowCount = rowsNeededToCoverViewport * 2;
            var rows = [];
            rows.length = desiredRowCount;
            for (var i = 0; i < desiredRowCount; i++) {
                rows[i] = new virtualRow();
            }
            return rows;
        };
        virtualGrid.prototype.initializeColumns = function () {
            // If the consumer code passed in some columns, use those.
            if (this.settings.columns && this.settings.columns.length > 0) {
                // Also, insert the row selection checkbox column if we're configured to do so.
                if (this.settings.showRowSelectionCheckbox) {
                    this.columns([new checkedColumn()].concat(this.settings.columns));
                }
                else {
                    this.columns(this.settings.columns);
                }
            }
        };
        virtualGrid.prototype.gridScrolled = function () {
            var _this = this;
            window.cancelAnimationFrame(this.scrollAnimationFrameHandle);
            this.scrollAnimationFrameHandle = window.requestAnimationFrame(function () { return _this.render(); });
        };
        virtualGrid.prototype.render = function () {
            var _this = this;
            // The grid may not be visible if the results returned quickly and we haven't finished initializing the UI.
            // In such a case, we queue up a render to occur later.
            if (this.checkGridVisibility()) {
                this.checkForUpdatedGridHeight();
                this.layoutVirtualRowPositions();
                this.fillDataIntoRows();
            }
            else {
                // Grid isn't yet visible. Queue up a render later.
                window.cancelAnimationFrame(this.renderHandle);
                this.renderHandle = window.requestAnimationFrame(function () { return _this.render(); });
            }
        };
        virtualGrid.prototype.findGridElement = function () {
            var element = $(document.querySelector("#" + this.gridId));
            if (element.length === 0) {
                throw new Error("Couldn't find grid element with ID " + this.gridId);
            }
            return element;
        };
        virtualGrid.prototype.fetchItems = function (skip, take) {
            var _this = this;
            if (this.isLoading()) {
                this.queuedFetch = { skip: skip, take: take };
            }
            else {
                this.isLoading(true);
                var safeSkip_1 = skip;
                var safeTake_1 = take;
                if (this.totalItemCount != null && skip > this.totalItemCount) {
                    skip = this.totalItemCount;
                }
                if (this.totalItemCount != null && (skip + take) > this.totalItemCount) {
                    safeTake_1 = this.totalItemCount - skip;
                }
                if (safeTake_1 > 0) {
                    this.settings.fetcher(safeSkip_1, safeTake_1)
                        .then(function (results) { return _this.chunkFetched(results, safeSkip_1, safeTake_1); })
                        .fail(function (error) { return _this.chunkFetchFailed(error, skip, safeTake_1); })
                        .always(function () {
                        // When we're done loading, run the next queued fetch as necessary.
                        _this.isLoading(false);
                        _this.runQueuedFetch();
                    });
                }
            }
        };
        virtualGrid.prototype.checkForUpdatedGridHeight = function () {
            var oldHeight = this.gridElementHeight;
            var newHeight = this.$gridElement.height();
            this.gridElementHeight = newHeight;
            // If the grid grew taller, we may need more virtual rows.
            if (newHeight > oldHeight) {
                this.initializeVirtualRows();
            }
            return newHeight;
        };
        virtualGrid.prototype.checkGridVisibility = function () {
            // If we've already determined the grid is visible, roll with that.
            if (this.isGridVisible) {
                return true;
            }
            else if (this.$gridElement) {
                return (this.isGridVisible = this.$gridElement.is(":visible"));
            }
            return false;
        };
        virtualGrid.prototype.runQueuedFetch = function () {
            if (this.queuedFetch) {
                var _a = this.queuedFetch, skip = _a.skip, take = _a.take;
                this.queuedFetch = null;
                // The previous fetch may have fetched some or all of the items we're about to fetch now.
                // So, before running the queued fetch, modify it to fetch the next chunk of unavailable items.
                var indexOfNextUnavailableChunk = skip;
                for (var i = skip; i < this.items.length; i++) {
                    if (!this.items[i]) {
                        indexOfNextUnavailableChunk = i;
                        break;
                    }
                }
                this.fetchItems(indexOfNextUnavailableChunk, take);
            }
        };
        virtualGrid.prototype.layoutVirtualRowPositions = function () {
            // This is hot path, called multiple times when scrolling. 
            // Keep allocations to a minimum.
            // Determine the view port.
            var scrollTop = this.$viewportElement.scrollTop();
            var scrollBottom = scrollTop + this.gridElementHeight;
            var positionCheck = scrollTop;
            var columns = this.columns();
            var lastPossibleRowY = this.virtualHeight() - virtualRow.height; // Find out the last possible row in the grid so that we don't place virtual rows beneath this.
            while (positionCheck < scrollBottom && positionCheck <= lastPossibleRowY) {
                var rowAtPosition = this.findRowAtY(positionCheck);
                if (!rowAtPosition) {
                    // There's no row at that spot. Find one we can put there.
                    rowAtPosition = this.getOffscreenRow(scrollTop, scrollBottom);
                    // Populate it with data.
                    var rowIndex = Math.floor(positionCheck / virtualRow.height);
                    var isChecked = this.selectedIndices.indexOf(rowIndex) !== -1;
                    rowAtPosition.populate(this.items[rowIndex], rowIndex, isChecked, columns);
                }
                var newPositionCheck = rowAtPosition.top + virtualRow.height;
                if (newPositionCheck <= positionCheck) {
                    throw new Error("Virtual grid defect: next position check was smaller or equal to last check, resulting in potentially infinite loop.");
                }
                positionCheck = newPositionCheck;
            }
        };
        virtualGrid.prototype.fillDataIntoRows = function () {
            // Find which rows are visible.
            var scrollTop = this.$viewportElement.scrollTop();
            var scrollBottom = scrollTop + this.gridElementHeight;
            var columns = this.columns();
            var firstVisibleRowIndex = null;
            var totalVisible = 0;
            for (var i = 0; i < this.virtualRows.length; i++) {
                var virtualRow_1 = this.virtualRows[i];
                var isVisible = !virtualRow_1.isOffscreen(scrollTop, scrollBottom);
                if (isVisible) {
                    firstVisibleRowIndex = firstVisibleRowIndex === null ? virtualRow_1.index : Math.min(virtualRow_1.index, firstVisibleRowIndex);
                    totalVisible++;
                    // Fill it with the data we've got loaded. If there's no data, it will display the loading indicator.
                    var isRowChecked = this.selectedIndices.indexOf(virtualRow_1.index) !== -1;
                    virtualRow_1.populate(this.items[virtualRow_1.index], virtualRow_1.index, isRowChecked, columns);
                }
            }
            // Of the visible rows, are we missing items for any of them?
            var needsToFetch = false;
            if (firstVisibleRowIndex !== null) {
                for (var i = firstVisibleRowIndex; i < firstVisibleRowIndex + totalVisible; i++) {
                    if (!this.items[i]) {
                        needsToFetch = true;
                        break;
                    }
                }
            }
            // We're missing items. Fetch them.
            if (needsToFetch) {
                // Take about 50 items before the first visible index (in case we're scrolling up)
                var skip = Math.max(0, firstVisibleRowIndex - 50);
                var take = Math.max(virtualGrid.minItemFetchCount, totalVisible);
                this.fetchItems(skip, take);
            }
        };
        virtualGrid.prototype.findRowAtY = function (y) {
            // This is hot path, called multiple times when scrolling. 
            // Keep allocations to a minimum.
            for (var i = 0; i < this.virtualRows.length; i++) {
                var vRow = this.virtualRows[i];
                var vRowTop = vRow.top;
                var vRowBottom = vRowTop + virtualRow.height;
                if (vRowTop <= y && vRowBottom > y) {
                    return vRow;
                }
            }
            return null;
        };
        virtualGrid.prototype.getOffscreenRow = function (viewportTop, viewportBottom) {
            // This is hot path, called multiple times when scrolling.
            // Keep allocations to a minimum.
            for (var i = 0; i < this.virtualRows.length; i++) {
                var row = this.virtualRows[i];
                if (row.isOffscreen(viewportTop, viewportBottom)) {
                    return row;
                }
            }
            throw new Error("Virtual grid defect: couldn't find an offscreen row to recycle. viewportTop = " + viewportTop + ", viewportBottom = " + viewportBottom + ", recycle row count = " + this.virtualRows.length);
        };
        virtualGrid.prototype.chunkFetchFailed = function (error, skip, take) {
            // Any rows displaying these items, show an error.
            var endIndex = skip + take;
            var failedRows = this.virtualRows
                .filter(function (r) { return !r.hasData && r.index >= skip && r.index <= endIndex; });
            failedRows.forEach(function (r) { return r.dataLoadError(error); });
        };
        virtualGrid.prototype.chunkFetched = function (results, skip, take) {
            if (!this.columns() || this.columns().length === 0) {
                this.assignColumnFromItems(results.items);
            }
            // Add these results to the .items array as necessary.
            this.items.length = results.totalResultCount;
            this.totalItemCount = results.totalResultCount;
            this.virtualHeight(results.totalResultCount * virtualRow.height);
            var endIndex = skip + results.items.length;
            for (var i = 0; i < results.items.length; i++) {
                var rowIndex = i + skip;
                this.items[rowIndex] = results.items[i];
            }
            this.render();
        };
        virtualGrid.prototype.assignColumnFromItems = function (items) {
            var propertySet = {};
            var itemPropertyNames = [].concat.apply([], items.map(function (i) { return Object.keys(i); }));
            var uniquePropertyNames = new Set(itemPropertyNames);
            var columnNames = Array.from(uniquePropertyNames);
            var viewportWidth = this.$viewportElement.prop("clientWidth");
            var checkedColumnWidth = this.settings.showRowSelectionCheckbox ? checkedColumn.columnWidth : 0;
            var columnWidth = Math.floor(viewportWidth / columnNames.length) - checkedColumnWidth + "px";
            // Put Id and Name columns first.
            var prioritizedColumns = ["Id", "Name"];
            prioritizedColumns
                .reverse()
                .forEach(function (c) {
                var columnIndex = columnNames.indexOf(c);
                if (columnIndex >= 0) {
                    columnNames.splice(columnIndex, 1);
                    columnNames.unshift(c);
                }
            });
            // Insert the row selection checkbox column as necessary.
            var initialColumns = this.settings.showRowSelectionCheckbox ? [new checkedColumn()] : [];
            this.columns(initialColumns.concat(columnNames.map(function (p) { return new textColumn(p, p, columnWidth); })));
        };
        virtualGrid.prototype.gridClicked = function (e) {
            if (e.target) {
                var $target = $(e.target);
                // If we clicked the the checked column header, toggle select all.
                if ($target.hasClass("checked-column-header")) {
                    this.isSelectAllChecked(!this.isSelectAllChecked());
                }
                else if ($target.hasClass("checked-cell-input")) {
                    // If we clicked a checked cell, toggle its selected state.
                    var rowIndex = this.findRowIndexForCell($target);
                    if (rowIndex !== null) {
                        this.toggleRowSelected(rowIndex);
                    }
                }
            }
        };
        virtualGrid.prototype.findRowIndexForCell = function (cellElement) {
            return this.virtualRows
                .filter(function (r) { return r.element.find(cellElement).length > 0; })
                .map(function (r) { return r.index; })[0];
        };
        /**
         * Clears the items from the grid and refetches the first chunk of items.
         */
        virtualGrid.prototype.resetItems = function () {
            this.items.length = 0;
            this.totalItemCount = null;
            this.queuedFetch = null;
            this.isLoading(false);
            this.virtualRows.forEach(function (r) { return r.reset(); });
            this.fetchItems(0, 100);
        };
        virtualGrid.prototype.toggleRowSelected = function (rowIndex) {
            var selectionIndex = this.selectedIndices.indexOf(rowIndex);
            if (selectionIndex === -1) {
                this.selectedIndices.push(rowIndex);
            }
            else {
                this.selectedIndices.splice(selectionIndex, 1);
            }
            // If we had all selected or none selected, remove that state.
            if (this.isSelectAllChecked() === true || this.isSelectAllChecked() === false) {
                this.isSelectAllChecked(null);
                $(".checked-column-header").prop("checked", false);
            }
            this.render();
        };
        /*
         * Handles when the "select all" observable changes, either by clicking the check column header or via code.
         */
        virtualGrid.prototype.selectAllChanged = function (allSelected) {
            // When selectAll is set to false, remove all selected indices.
            if (allSelected === false) {
                this.selectedIndices.length = 0;
            }
            else if (allSelected === true) {
                this.selectedIndices.length = this.items.length;
                for (var i = 0; i < this.selectedIndices.length; i++) {
                    this.selectedIndices[i] = i;
                }
            }
            // Note: allSelected can be set to null, meaning some items are checked while others are not.
            this.render();
        };
        /**
         * Installs the virtual grid component if it's not already installed.
         */
        virtualGrid.install = function () {
            var componentName = "virtual-grid";
            if (!ko.components.isRegistered(componentName)) {
                // TODO: we may want to move this to an HTML file that's fetched with RequireJS. Knockout components do support this.
                ko.components.register(componentName, {
                    viewModel: virtualGrid,
                    template: "\n<div class=\"virtual-grid\" data-bind=\"attr: { id: gridId }\">\n    <!-- Columns -->\n    <div class=\"column-container\" data-bind=\"foreach: columns, visible: settings.showColumns\"><div class=\"column\" data-bind=\"style: { width: $data.width }\"><strong data-bind=\"html: $data.display\"></strong></div></div>    \n\n    <!-- Viewport -->\n    <!-- The viewport is the section of the grid showing visible rows -->\n    <div class=\"viewport\" data-bind=\"css: { 'columns-visible': settings.showColumns }\">\n\n        <!-- The viewport scroller is the very tall scrolling part -->\n        <div class=\"viewport-scroller\" data-bind=\"style: { height: virtualHeight() + 'px' }, template: { afterRender: afterRender.bind($data) }\">\n\n        </div>\n    </div>\n    \n</div>\n"
                });
            }
        };
        return virtualGrid;
    }());
    virtualGrid.minItemFetchCount = 100;
    virtualGrid.viewportSelector = ".viewport";
    virtualGrid.viewportScrollerSelector = ".viewport-scroller";
    return virtualGrid;
});

//# sourceMappingURL=virtualGrid.js.map
