define(["require", "exports"], function (require, exports) {
    "use strict";
    /**
     * A virtual row. Contains an element displayed as a row in the grid. Gets recycled as the grid scrolls in order to create and manage fewer elements.
     */
    var VirtualRow = (function () {
        function VirtualRow() {
            this.item = null; // The last item populated into this virtual row.
            this.isItemSelected = false;
            this._top = -9999;
            this._index = -1;
            this.even = true;
            this.element = $("<div class=\"virtual-row\" style=\"height: " + VirtualRow.height + "px; top: " + this.top + "px\"></div>");
        }
        Object.defineProperty(VirtualRow.prototype, "top", {
            get: function () {
                return this._top;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VirtualRow.prototype, "index", {
            /**
             * Gets the index of the row this virtual row is displaying.
             */
            get: function () {
                return this._index;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(VirtualRow.prototype, "hasData", {
            get: function () {
                return !!this.item;
            },
            enumerable: true,
            configurable: true
        });
        VirtualRow.prototype.isOffscreen = function (scrollTop, scrollBottom) {
            var top = this.top;
            var bottom = top + VirtualRow.height;
            return top > scrollBottom || bottom < scrollTop;
        };
        VirtualRow.prototype.dataLoadError = function (error) {
            this.element.text("Unable to load data: " + JSON.stringify(error));
        };
        VirtualRow.prototype.populate = function (item, rowIndex, isSelected, columns) {
            // Optimization: don't regenerate this row HTML if nothing's changed since last render.
            var alreadyDisplayingData = !!item && this.item === item && this._index === rowIndex && this.isItemSelected === isSelected;
            if (!alreadyDisplayingData) {
                this.item = item;
                this._index = rowIndex;
                // If we have data, fill up this row content.
                if (item) {
                    var html = this.createCellsHtml(item, columns, isSelected);
                    this.element.html(html);
                }
                else {
                    this.element.text("Loading...");
                }
                // Update the selected status. Displays as a different row color.
                var hasChangedSelectedStatus = this.isItemSelected !== isSelected;
                if (hasChangedSelectedStatus) {
                    this.element.toggleClass("selected");
                    this.isItemSelected = isSelected;
                }
                // Update the "even" status. Used for striping the virtual rows.
                var hasChangedEven = this.even !== (rowIndex % 2 === 0);
                if (hasChangedEven) {
                    this.element.toggleClass("even");
                    this.even = !this.even;
                }
                // Move it to its proper position.
                var desiredNewRowY = rowIndex * VirtualRow.height;
                this.setElementTop(desiredNewRowY);
            }
        };
        VirtualRow.prototype.reset = function () {
            this.item = null;
            this.isItemSelected = false;
            this.setElementTop(-9999);
            this._index = -1;
            this.even = true;
            this.element.text("");
        };
        VirtualRow.prototype.createCellsHtml = function (item, columns, isSelected) {
            var cellsHtml = columns
                .map(function (c) { return "<div class=\"cell " + c.cellClass + "\" style=\"width: " + c.width + "\">" + c.renderCell(item, isSelected) + "</div>"; });
            return cellsHtml.join("");
        };
        VirtualRow.prototype.setElementTop = function (val) {
            this._top = val;
            this.element.css("top", val);
        };
        return VirtualRow;
    }());
    VirtualRow.height = 36;
    return VirtualRow;
});

//# sourceMappingURL=virtualRow.js.map
