define(["require", "exports", "widgets/virtualTable/cell", "common/execJs"], function (require, exports, cell, execJs) {
    "use strict";
    var row = (function () {
        function row(addIdCell, viewModel) {
            this.viewModel = viewModel;
            this.top = ko.observable(0);
            this.rowIndex = ko.observable(0);
            this.isInUse = ko.observable(false);
            this.cellMap = { Id: null };
            this.collectionClass = ko.observable("");
            this.editUrl = ko.observable("");
            this.isChecked = ko.observable(false);
            this.compiledCustomFunctions = {};
            this.templateNameCache = {};
            if (addIdCell) {
                this.addOrUpdateCellMap("Id", null);
            }
            this.viewModel.settings.customFunctions.subscribe(this.extractCustomFunctions);
            this.extractCustomFunctions(this.viewModel.settings.customFunctions());
        }
        row.prototype.calculateExternalIdCellColor = function (cellValue) {
            var cellCollectionName = cellValue.slice(0, cellValue.lastIndexOf("/")).toLocaleLowerCase();
            var matchingCollection = this.viewModel.settings.collections().find(function (c) { return c.name.toLocaleLowerCase() === cellCollectionName; });
            if (!!matchingCollection) {
                return matchingCollection.colorClass;
            }
            return "";
        };
        row.prototype.calculateExternalGroupCellColor = function (cellValue) {
            var matchingGroup = this.viewModel.settings.collections().find(function (c) { return c.name === cellValue; });
            if (!!matchingGroup) {
                return matchingGroup.colorClass;
            }
            return "";
        };
        row.prototype.extractCustomFunctions = function (newValue) {
            this.compiledCustomFunctions = new Function("var exports = {}; " + newValue.functions + "; return exports;")();
        };
        row.prototype.resetCells = function () {
            for (var prop in this.cellMap) {
                var cellVal = this.cellMap[prop];
                if (cellVal) {
                    cellVal.reset();
                }
            }
            this.collectionClass("");
            this.isChecked(false);
        };
        row.prototype.createPlaceholderCells = function (cellNames) {
            var _this = this;
            cellNames
                .filter(function (c) { return c != "Id"; })
                .forEach(function (c) { return _this.addOrUpdateCellMap(c, null); });
        };
        row.prototype.fillCells = function (rowData) {
            var _this = this;
            var customColumns = this.viewModel.settings.customColumns();
            this.isInUse(true);
            var rowProperties = rowData.getDocumentPropertyNames();
            if (customColumns.customMode()) {
                customColumns.columns().forEach(function (column, index) {
                    var binding = column.binding();
                    var context = {};
                    $.each(rowData, function (name, value) {
                        context[name] = value;
                    });
                    for (var p in _this.compiledCustomFunctions) {
                        context[p] = _this.compiledCustomFunctions[p];
                    }
                    var cellValue = execJs.createSimpleCallableCode("return " + binding + ";", context)();
                    var callValueAsString = (typeof cellValue === "object" && _this.getCellTemplateName(binding, rowData) !== cell.customTemplate) ? JSON.stringify(cellValue, null, 4) : cellValue;
                    _this.addOrUpdateCellMap(binding, callValueAsString);
                });
            }
            else {
                for (var i = 0; i < rowProperties.length; i++) {
                    var prop = rowProperties[i];
                    var cellValue = rowData[prop];
                    // pass json object when not custom template!
                    if (typeof cellValue === "object" && this.getCellTemplateName(prop, rowData) !== cell.customTemplate) {
                        cellValue = JSON.stringify(cellValue, null, 4) || "";
                    }
                    if (cellValue && cellValue.length > 300) {
                        cellValue = cellValue.substring(0, 300);
                    }
                    this.addOrUpdateCellMap(prop, cellValue);
                }
            }
            if (rowData.getId()) {
                this.addOrUpdateCellMap("Id", rowData.getId());
            }
        };
        row.prototype.addOrUpdateCellMap = function (propertyName, data) {
            if (!this.cellMap[propertyName]) {
                this.cellMap[propertyName] = new cell(data, this.getCellTemplateName(propertyName, data));
            }
            else {
                var cellVal = this.cellMap[propertyName];
                cellVal.update(data);
            }
            var cacheKey = this.getOrAddTemplateNameCache(propertyName);
            cacheKey(this.getCellTemplate(propertyName));
        };
        row.prototype.getCellData = function (cellName) {
            var cellVal = this.cellMap[cellName];
            if (cellVal) {
                return cellVal.data;
            }
            return "";
        };
        row.prototype.getOrAddTemplateNameCache = function (cellName) {
            var cacheKey = this.templateNameCache[cellName];
            if (!cacheKey) {
                cacheKey = this.templateNameCache[cellName] = ko.observable("nullTemplate");
            }
            return cacheKey;
        };
        row.prototype.getCellTemplate = function (cellName) {
            var cellVal = this.cellMap[cellName];
            if (cellVal) {
                if (cellVal.resetFlag) {
                    cellVal.templateName = this.getCellTemplateName(cellName, this.cellMap[cellName].data());
                    cellVal.resetFlag = false;
                    return cellVal.templateName;
                }
                else {
                    return cellVal.templateName;
                }
            }
            // Bug fix: http://issues.hibernatingrhinos.com/issue/RavenDB-2002
            // Calling .data() registers it as a Knockout dependency; updating this 
            // observable later will cause the cell to redraw, thus fixing the bug.
            if (this.cellMap && this.cellMap["Id"]) {
                this.cellMap["Id"].data();
            }
            return "nullTemplate";
        };
        row.prototype.getCellTemplateName = function (propertyName, data) {
            if (propertyName === "Id") {
                return cell.idTemplate;
            }
            else if (propertyName === "__IsChecked") {
                return cell.checkboxTemplate;
            }
            switch (this.viewModel.settings.viewType) {
                case 1 /* Counters */:
                    if (propertyName === "Counter Name") {
                        return cell.counterNameTemplate;
                    }
                    else if (this.viewModel.settings.isCounterAllGroupsGroup() && propertyName === "Group Name") {
                        return cell.counterGroupTemplate;
                    }
                    return cell.defaultTemplate;
                case 2 /* TimeSeries */:
                    if (propertyName === "Key") {
                        return cell.timeSeriesKeyTemplate;
                    }
                    return cell.defaultTemplate;
            }
            // see if this is an ID or external ID cell.
            if (!!data) {
                if (typeof data == "string") {
                    var cleanData = data.replace('/\t+/g', '')
                        .replace(/\s+/g, '')
                        .replace('/\n+/g', '');
                    if (/^\[{"[a-zA-Z0-9_-]+":/.test(cleanData) ||
                        //this handy REGEX for testing URLs was taken from http://stackoverflow.com/questions/8188645/javascript-regex-to-match-a-url-in-a-field-of-text
                        /(http|ftp|https):\/\/[\w-]+(\.[\w-]+)+([\w.,@?^=%&amp;:\/~+#-]*[\w@?^=%&amp;\/~+#-])?/.test(cleanData))
                        return cell.defaultTemplate;
                    if (/^\w+\/\w+/ig.test(data) && this.viewModel.collectionExists(data)) {
                        return cell.externalIdTemplate;
                    }
                }
                else if (!!data[propertyName] &&
                    typeof data[propertyName] == "string" &&
                    /\w+\/\w+/.test(data[propertyName])) {
                    return cell.externalIdTemplate;
                }
            }
            // note: we just inform here about custom template - without specific name of this template.
            var colParam = this.viewModel.settings.customColumns().findConfigFor(propertyName);
            if (colParam && colParam.template() !== cell.defaultTemplate && colParam.template() !== cell.counterGroupTemplate) {
                return cell.customTemplate;
            }
            return cell.defaultTemplate;
        };
        return row;
    }());
    return row;
});

//# sourceMappingURL=row.js.map
